@@ -18,36 +18,35 @@
 #ifndef MAINWINDOW_H
 #define MAINWINDOW_H
 
+#include <config-amarok.h>
+
 #include "amarok_export.h"
-#include "meta/Meta.h"
-#include "EngineObserver.h"
-#include "browsers/BrowserWidget.h"
+#include "core/meta/Meta.h"
+#include "browsers/BrowserDock.h"
 
 #include <KMainWindow>
 #include <KVBox>
+#include <Phonon/Global>
+#include <Phonon/Global>
 
-#include <QPointer>
+#include <QWeakPointer>
 
 class CollectionWidget;
-class ContextWidget;
 class SlimToolbar;
 class MainToolbar;
 class MainWindow;
+#ifdef DEBUG_BUILD_TYPE
+class NetworkAccessViewer;
+#endif // DEBUG_BUILD_TYPE
 class PlaylistFileProvider;
-class SearchWidget;
 
 namespace PlaylistBrowserNS { class PlaylistBrowser; }
+namespace Playlist { class Dock; }
+class ContextDock;
 
-namespace Plasma { class Containment; }
-namespace Playlist { class Widget; }
-
-namespace Context {
-    class ContextScene;
-    class ContextView;
-    class ToolbarView;
-}
 
 class KMenu;
+class KAction;
 class QMenuBar;
 class QSplitter;
 class QTimer;
@@ -65,47 +64,47 @@ namespace The {
   *
   * This is the main window widget.
   */
-class AMAROK_EXPORT MainWindow : public KMainWindow, public EngineObserver, public Meta::Observer
+class AMAROK_EXPORT MainWindow : public KMainWindow
 {
     friend MainWindow* The::mainWindow();
 
     Q_OBJECT
 
     public:
+        enum AmarokDockId {
+            AmarokDockNavigation,
+            AmarokDockContext,
+            AmarokDockPlaylist
+        };
+        
         MainWindow();
         ~MainWindow();
 
-        //allows us to switch browsers from within other browsers etc
-        void showBrowser( const QString& name );
-        void addBrowser( const QString &name, QWidget *widget, const QString &text, const QString &icon );
-
-        //takes into account minimized, multiple desktops, etc.
-        bool isReallyShown() const;
-
         void activate();
 
-        BrowserWidget *browserWidget() const { return m_browsers; }
-        QPointer<KMenu> ToolsMenu() const { return m_toolsMenu; }
-        QPointer<KMenu> SettingsMenu() const { return m_settingsMenu; }
-        QPointer<Playlist::Widget> playlistWidget() { return m_playlistWidget; }
+        //allows us to switch browsers from within other browsers etc
+        void showBrowser( const QString& name );
+
+        //ensures the dock widget is visible in case it is tabbed
+        void showDock( AmarokDockId dockId );
+
+        BrowserDock *browserDock() const { return m_browserDock.data(); }
+        KMenu *ToolsMenu() const { return m_toolsMenu.data(); }
+        KMenu *SettingsMenu() const { return m_settingsMenu.data(); }
+        Playlist::Dock *playlistDock() const { return m_playlistDock.data(); }
         void deleteBrowsers();
 
         /* Reimplemented from QMainWindow to allow only one active toolbar at any time */
         virtual QMenu* createPopupMenu();
 
+        void addViewMenuItems(QMenu* menu);
+
         QString activeBrowserName();
 
         CollectionWidget * collectionBrowser();
         PlaylistBrowserNS::PlaylistBrowser * playlistBrowser();
 
-        //will return the size of the rect defined top, right and left by the main toolbar and bottom by the context view.
-        QSize backgroundSize();
-
-        int contextXOffset();
-        QRect contextRectGlobal();
-        QPoint globalBackgroundOffset();
-
-        bool isLayoutLocked();
+        bool isLayoutLocked() const;
 
         /**
         *    If an audiocd collection is present. Stop current playback, clear playlist,
@@ -113,7 +112,7 @@ class AMAROK_EXPORT MainWindow : public KMainWindow, public EngineObserver, publ
         */
         bool playAudioCd();
 
-        bool isWaitingForCd();
+        bool isWaitingForCd() const;
 
     signals:
         void loveTrack( Meta::TrackPtr track );
@@ -124,36 +123,32 @@ class AMAROK_EXPORT MainWindow : public KMainWindow, public EngineObserver, publ
     public slots:
         void showHide();
         void slotFullScreen();
-        void slotLoveTrack() { emit loveTrack( The::engineController()->currentTrack() ); }
+        void slotLoveTrack();
         void showNotificationPopup();
-        void hideContextView( bool hide );
-
         void setLayoutLocked( bool locked );
-
         void showAbout();
-
-    protected:
-        //Reimplemented from EngineObserver
-        virtual void engineStateChanged( Phonon::State state, Phonon::State oldState = Phonon::StoppedState );
-        virtual void engineNewTrackPlaying();
-
-        //Reimplemented from Meta::Observer
-        using Observer::metadataChanged;
-        virtual void metadataChanged( Meta::TrackPtr track );
+        void showReportBug();
 
     private slots:
+        void slotStopped();
+        void slotPaused();
+        void slotNewTrackPlaying();
+        void slotMetadataChanged( Meta::TrackPtr track );
+
         void exportPlaylist() const;
         void slotShowActiveTrack() const;
         void slotShowBookmarkManager() const;
         void slotShowEqualizer() const;
         void slotShowCoverManager() const;
+        void slotShowMenuBar();
         void slotPlayMedia();
         void slotAddLocation( bool directPlay = false );
         void slotAddStream();
         void slotJumpTo();
         void showScriptSelector();
-        void layoutChanged();
-        void ignoreLayoutChangesTimeout();
+#ifdef DEBUG_BUILD_TYPE
+        void showNetworkRequestViewer();
+#endif // DEBUG_BUILD_TYPE
 
         /**
          * Save state and position of dock widgets.
@@ -167,9 +162,8 @@ class AMAROK_EXPORT MainWindow : public KMainWindow, public EngineObserver, publ
 
     protected:
         virtual void closeEvent( QCloseEvent* );
-        virtual void keyPressEvent( QKeyEvent* );
-        virtual void resizeEvent ( QResizeEvent * event );
         virtual void paletteChange( const QPalette & oldPalette );
+        virtual bool queryExit(); 
 
     private slots:
         void setRating1() { setRating( 1 ); }
@@ -181,63 +175,38 @@ class AMAROK_EXPORT MainWindow : public KMainWindow, public EngineObserver, publ
     private:
         void init();
         void setRating( int n );
-        void showBrowser( const int index );
 
         CollectionWidget * m_collectionBrowser;
         PlaylistBrowserNS::PlaylistBrowser * m_playlistBrowser;
 
-        QPointer<QMenuBar>  m_menubar;
-        QPointer<KMenu>     m_toolsMenu;
-        QPointer<KMenu>     m_settingsMenu;
-        QPointer<BrowserWidget>   m_browsers;
-        QStringList         m_browserNames;
-        QPointer<KMenu>     m_searchMenu;
-        //QPointer<KVBox>     m_statusbarArea;
+        QWeakPointer<QMenuBar>  m_menubar;
+        QWeakPointer<KMenu>     m_toolsMenu;
+        QWeakPointer<KMenu>     m_settingsMenu;
+        QWeakPointer<BrowserDock> m_browserDock;
+#ifdef DEBUG_BUILD_TYPE
+        QWeakPointer<NetworkAccessViewer> m_networkViewer;
+#endif // DEBUG_BUILD_TYPE
 
-        QPointer<SearchWidget>     m_searchWidget;
-        QPointer<Playlist::Widget> m_playlistWidget;
-        QPointer<QTimer>           m_timer;  //search filter timer
-        QPointer<QSplitter>        m_splitter;
+        QByteArray m_splitterState;
 
-        QByteArray                 m_splitterState;
+        QWeakPointer<ContextDock> m_contextDock;
+        QWeakPointer<Playlist::Dock> m_playlistDock;
 
-        QPointer<ContextWidget>         m_contextWidget;
-        QPointer<Context::ContextScene> m_corona;
-        QPointer<Context::ContextView>  m_contextView;
-        QPointer<Context::ToolbarView>  m_contextToolbarView;
+        QWeakPointer<SlimToolbar> m_slimToolbar;
+        QWeakPointer<MainToolbar> m_mainToolbar;
 
-        Meta::TrackPtr m_currentTrack;
+        void createActions();
+        void createMenus();
 
-        QDockWidget * m_browsersDock;
-        QDockWidget * m_contextDock;
-        QDockWidget * m_playlistDock;
+        KAction* m_showMenuBar;
 
-        QPointer<SlimToolbar> m_slimToolbar;
-        QPointer<MainToolbar> m_mainToolbar;
+        int m_lastBrowser;
+        int m_searchField;
 
-        QWidget *     m_browserDummyTitleBarWidget;
-        QWidget *     m_contextDummyTitleBarWidget;
-        QWidget *     m_playlistDummyTitleBarWidget;
-
-
-        void    createActions();
-        void    createMenus();
-        int     m_lastBrowser;
-        int     m_searchField;
-
-        static QPointer<MainWindow> s_instance;
+        static QWeakPointer<MainWindow> s_instance;
 
         bool m_layoutLocked;
-        bool m_dockWidthsLocked;
-        bool m_dockChangesIgnored;
-        QTimer * m_restoreLayoutTimer;
-        QTimer * m_ignoreLayoutChangesTimer;
-        QTimer * m_saveLayoutChangesTimer;
-
         bool m_waitingForCd;
-
-    private slots:
-        void createContextView( Plasma::Containment *c );
 };
 
 
