@@ -13,12 +13,13 @@
  * You should have received a copy of the GNU General Public License along with         *
  * this program.  If not, see <http://www.gnu.org/licenses/>.                           *
  ****************************************************************************************/
+
+#define DEBUG_PREFIX "PaletteHandler"
  
 #include "PaletteHandler.h"
 
 #include "App.h"
-#include "Debug.h"
-#include "MainWindow.h"
+#include "core/support/Debug.h"
 
 #include <kglobal.h>
 
@@ -60,15 +61,40 @@ PaletteHandler::updateItemView( QAbstractItemView * view )
 {
     QPalette p = m_palette;
 
-    QColor c = p.color( QPalette::Base );
-    c.setAlpha( 0 );
-    p.setColor( QPalette::Base, c );
-
-    c = p.color( QPalette::AlternateBase );
+    QColor c = p.color( QPalette::AlternateBase );
     c.setAlpha( 77 );
     p.setColor( QPalette::AlternateBase, c );
-
+    // to fix hardcoded QPalette::Text usage in Qt classes
+    p.setColor( QPalette::Base, p.color(QPalette::Window) );
+    p.setColor( QPalette::Text, p.color(QPalette::WindowText) );
     view->setPalette( p );
+    
+    if ( QWidget *vp = view->viewport() )
+    {
+        // don't paint background - do NOT use Qt::transparent etc.
+        vp->setAutoFillBackground( false );
+        vp->setBackgroundRole( QPalette::Window );
+        vp->setForegroundRole( QPalette::WindowText );
+        // erase custom viewport palettes, shall be "transparent"
+        vp->setPalette(QPalette());
+    }
+}
+
+QColor
+PaletteHandler::foregroundColor( const QPainter *p, bool selected )
+{
+    QPalette pal;
+    QPalette::ColorRole fg = QPalette::WindowText;
+    if ( p->device() && p->device()->devType() == QInternal::Widget)
+    {
+        QWidget *w = static_cast<QWidget*>( p->device() );
+        fg = w->foregroundRole();
+        pal = w->palette();
+    }
+    else
+        pal = App::instance()->palette();
+
+    return pal.color( selected ? QPalette::HighlightedText : fg );
 }
 
 QPalette
@@ -96,6 +122,8 @@ PaletteHandler::highlightColor( qreal saturationPercent, qreal valuePercent )
     saturation *= saturationPercent;
     qreal value = highlight.valueF();
     value *= valuePercent;
+    if( value > 1.0 )
+        value = 1.0;
     highlight.setHsvF( highlight.hueF(), saturation, value, highlight.alphaF() );
 
     return highlight;
