@@ -18,9 +18,9 @@
 #ifndef COLLECTIONTREEITEM_H
 #define COLLECTIONTREEITEM_H
 
-#include "Collection.h"
+#include "core/collections/Collection.h"
 
-#include "meta/Meta.h"
+#include "core/meta/Meta.h"
 
 #include <QList>
 
@@ -33,7 +33,9 @@ namespace CustomRoles
         ByLineRole = Qt::UserRole + 3,
         HasCapacityRole = Qt::UserRole + 4,
         UsedCapacityRole = Qt::UserRole + 5,
+        /** The number of collection actions */
         DecoratorRoleCount = Qt::UserRole + 6,
+        /** The collection actions */
         DecoratorRole = Qt::UserRole + 7
     };
 }
@@ -43,7 +45,7 @@ class CollectionTreeItemModelBase;
 class CollectionTreeItem : public QObject
 {
     Q_OBJECT
-    Q_ENUMS( Type );
+    Q_ENUMS( Type )
 
     public:
         enum Type
@@ -51,14 +53,15 @@ class CollectionTreeItem : public QObject
             Root,
             Collection,
             VariousArtist,
+            NoLabel,
             Data
         };
 
         CollectionTreeItem( CollectionTreeItemModelBase *model ); //root node
         CollectionTreeItem( Meta::DataPtr data, CollectionTreeItem *parent, CollectionTreeItemModelBase *model  ); //data node
-        CollectionTreeItem( Amarok::Collection *parentCollection, CollectionTreeItem *parent, CollectionTreeItemModelBase *model  ); //collection node
-        //this ctor creates a "Various Artists" node. do not use it for anything else
-        CollectionTreeItem( const Meta::DataList &data, CollectionTreeItem *parent, CollectionTreeItemModelBase *model  ); //various artist node
+        CollectionTreeItem( Collections::Collection *parentCollection, CollectionTreeItem *parent, CollectionTreeItemModelBase *model  ); //collection node
+        //this ctor creates a "Various Artists" and "No Labels" nodes. do not use it for anything else
+        CollectionTreeItem( Type type, const Meta::DataList &data, CollectionTreeItem *parent, CollectionTreeItemModelBase *model  ); //various artist node
 
         ~CollectionTreeItem();
 
@@ -83,13 +86,17 @@ class CollectionTreeItem : public QObject
         bool isAlbumItem() const;
         bool isTrackItem() const;
         bool isVariousArtistItem() const;
+        bool isNoLabelItem() const;
 
-        QueryMaker* queryMaker() const;
+        Collections::QueryMaker* queryMaker() const;
+
+        /** Call addMatch for this objects data and it's query maker */
+        void addMatch( Collections::QueryMaker *qm ) const;
 
         bool operator<( const CollectionTreeItem& other ) const;
 
         const Meta::DataPtr data() const { return m_data; }
-        Amarok::Collection* parentCollection() const { return m_parentCollection; }
+        Collections::Collection* parentCollection() const { return m_parentCollection ? m_parentCollection : (m_parent ? m_parent->parentCollection() : 0); }
 
         KUrl::List urls() const;
         QList<Meta::TrackPtr> descendentTracks();
@@ -110,14 +117,16 @@ class CollectionTreeItem : public QObject
         void collectionUpdated();
 
     private:
-        QString albumYear() const;
+        /** Returns a list of collection actions.
+            Collection actions are shown on top of the collection tree item as icons (decorations)
+        */
         QList<QAction*> decoratorActions() const;
         void prepareForRemoval();
 
         Meta::DataPtr m_data;
         CollectionTreeItem *m_parent;
         CollectionTreeItemModelBase *m_model;
-        Amarok::Collection *m_parentCollection;
+        Collections::Collection *m_parentCollection;
 
         QList<CollectionTreeItem*> m_childItems;
         bool m_updateRequired;
