@@ -23,21 +23,22 @@
 
 QtGroupingProxy::QtGroupingProxy( QAbstractItemModel *model, QModelIndex rootNode, int groupedColumn )
     : QAbstractProxyModel()
-    , m_model( model )
     , m_rootNode( rootNode )
-    , m_folderIcon( QVariant( QIcon( "folder" ) ) )
+    , m_groupedColumn( 0 )
 {
+    setSourceModel( model );
+
     // signal proxies
-    connect( m_model,
+    connect( sourceModel(),
         SIGNAL( dataChanged( const QModelIndex&, const QModelIndex& ) ),
         this, SLOT( modelDataChanged( const QModelIndex&, const QModelIndex& ) )
     );
-    connect( m_model,
+    connect( sourceModel(),
         SIGNAL( rowsInserted( const QModelIndex&, int, int ) ), this,
         SLOT( modelRowsInserted( const QModelIndex &, int, int ) ) );
-    connect( m_model, SIGNAL( rowsRemoved( const QModelIndex&, int, int ) ),
+    connect( sourceModel(), SIGNAL( rowsRemoved( const QModelIndex&, int, int ) ),
         this, SLOT( modelRowsRemoved( const QModelIndex&, int, int ) ) );
-    connect( m_model, SIGNAL(layoutChanged()), SLOT(buildTree()) );
+    connect( sourceModel(), SIGNAL(layoutChanged()), SLOT(buildTree()) );
 
     if( m_groupedColumn != -1 )
         setGroupedColumn( groupedColumn );
@@ -54,69 +55,72 @@ QtGroupingProxy::setGroupedColumn( int groupedColumn )
     buildTree();
 }
 
-QList<ColumnVariantMap>
+QList<RowData>
 QtGroupingProxy::belongsTo( const QModelIndex &idx )
 {
     //qDebug() << __FILE__ << __FUNCTION__;
-    QList<ColumnVariantMap> cvmList;
+    QList<RowData> rowDataList;
 
-    //get all the data we have for this index
-    RoleVariantMap roleVariantMap = m_model->itemData( idx );
-    QMapIterator<int, QVariant> i(roleVariantMap);
+    //get all the data for this index from the model
+    IndexData indexData = sourceModel()->itemData( idx );
+    QMapIterator<int, QVariant> i( indexData );
     while( i.hasNext() )
     {
         i.next();
         int role = i.key();
         QVariant variant = i.value();
-        qDebug() << "role " << role << " : (" << variant.typeName() << ") : "<< variant; //so spammy
+        // qDebug() << "role " << role << " : (" << variant.typeName() << ") : "<< variant;
         if( variant.type() == QVariant::List )
         {
             //a list of variants get's expanded to multiple rows
             QVariantList list = variant.toList();
             for( int i = 0; i < list.length(); i++ )
             {
-                //take an existing CVM or create a new one
-                ColumnVariantMap cvm = (cvmList.count() > i) ?  cvmList.takeAt( i )
-                                       : ColumnVariantMap();
+                //take an existing row data or create a new one
+                RowData rowData = (rowDataList.count() > i) ?  rowDataList.takeAt( i )
+                                       : RowData();
 
                 //we only gather data for the first column
-                RoleVariantMap rvm = cvm.contains( 0 ) ? cvm.take( 0 ) : RoleVariantMap();
-                rvm.insert( role, list.value( i ) );
-                cvm.insert( 0, rvm );
-                cvmList.insert( i, cvm );
+                IndexData indexData = rowData.contains( 0 ) ? rowData.take( 0 ) : IndexData();
+                indexData.insert( role, list.value( i ) );
+                rowData.insert( 0, indexData );
+                //for the grouped column the data should not be gathered from the children
+                //this will allow filtering on the content of this column with a QSFP
+                rowData.insert( m_groupedColumn, indexData );
+                rowDataList.insert( i, rowData );
             }
         }
         else if( !variant.isNull() )
         {
             //it's just a normal item. Copy all the data and break this loop.
-            ColumnVariantMap cvm;
-            cvm.insert( 0, roleVariantMap );
-            cvmList << cvm;
+            RowData rowData;
+            rowData.insert( 0, indexData );
+            rowDataList << rowData;
             break;
         }
     }
     //for normal items (not root node) an empty list here means it's supposed to go in root.
-    if( cvmList.isEmpty() && idx != m_rootNode )
-        cvmList << ColumnVariantMap();
+    if( idx != m_rootNode && rowDataList.isEmpty() )
+        rowDataList << RowData();
 
-    return cvmList;
+    return rowDataList;
 }
 
 /* m_groupHash layout
 *  key : index of the group in m_groupMaps
-*  values : original row in m_model of children of this group
+*  value : a QList of the original rows in sourceModel() for the children of this group
 *
-*  key = -1  contains non-grouped indexes
+*  key = -1  contains a QList of the non-grouped indexes
 *
 * TODO: sub-groups
 */
 void
 QtGroupingProxy::buildTree()
 {
-    if( !m_model )
+    if( !sourceModel() )
         return;
 
-    emit layoutAboutToBeChanged();
+    beginResetModel();
 
     m_groupHash.clear();
     //don't clear the data maps since most of it will probably be needed again.
@@ -124,69 +128,107 @@ QtGroupingProxy::buildTree()
 
     //first load empty groups from the rootnode.
     QModelIndex rootGroupedIndex =
-            m_model->index( m_rootNode.row(), m_groupedColumn, m_rootNode.parent() );
+            sourceModel()->index( m_rootNode.row(), m_groupedColumn, m_rootNode.parent() );
     if( rootGroupedIndex.column() == m_groupedColumn )
     {
-        QList<ColumnVariantMap> groupData = belongsTo( rootGroupedIndex );
-        foreach( ColumnVariantMap cvm , groupData )
+        QList<RowData> groupData = belongsTo( rootGroupedIndex );
+        foreach( RowData rowData , groupData )
         {
-            qDebug() << cvm;
-            if( cvm.contains( 0 ) && cvm[0].contains( Qt::DisplayRole ) )
+            // qDebug() << rowData;
+            if( rowData.contains( 0 ) && rowData[0].contains( Qt::DisplayRole ) )
             {
-                QString groupName = cvm[0][Qt::DisplayRole].toString();
-                qDebug() << "Creating empty group: " << groupName;
-                m_groupMaps << cvm;
+                // qDebug() << "Creating empty group: " << rowData[0][Qt::DisplayRole].toString();
+                m_groupMaps << rowData;
             }
         }
     }
 
-    int max = m_model->rowCount( m_rootNode );
+    int max = sourceModel()->rowCount( m_rootNode );
     //qDebug() << QString("building tree with %1 leafs.").arg( max );
-    for( int row = max-1; row >= 0; row-- )
+    //WARNING: these have to be added in order because the addToGroups function is optimized for
+    //modelRowsInserted(). Failure to do so will result in wrong data shown in the view at best.
+    for( int row = 0; row < max; row++ )
     {
-        QModelIndex idx = m_model->index( row, m_groupedColumn, m_rootNode );
-        QList<ColumnVariantMap> groupData = belongsTo( idx );
-
-        //an item can be in multiple groups
-        foreach( ColumnVariantMap data, groupData )
-        {
-            int groupIndex = -1;
-            if( !data.isEmpty() )
-            {
-                QString groupName = data[0][Qt::DisplayRole].toString();
-                qDebug() << QString("index %1 belongs to group %2").arg( row ).arg( groupName );
-
-                foreach( const ColumnVariantMap &cachedData, m_groupMaps )
-                {
-                    //when this matches the index belongs to an existing group
-                    if( data[0][Qt::DisplayRole] == cachedData[0][Qt::DisplayRole] )
-                    {
-                        data = cachedData;
-                        break;
-                    }
-                }
-
-                groupIndex = m_groupMaps.indexOf( data );
-                //-1 means not found
-                if( groupIndex == -1 )
-                {
-                    //new groups are added to the end of the existing list
-                    m_groupMaps << data;
-                    groupIndex = m_groupMaps.count() - 1;
-                }
-            }
-
-            m_groupHash.insertMulti( groupIndex, row );
-        }
+        QModelIndex idx = sourceModel()->index( row, m_groupedColumn, m_rootNode );
+        addSourceRow( idx );
     }
-    dumpGroups();
+//    dumpGroups();
 
-    emit layoutChanged();
+    endResetModel();
 }
 
-/** Each Modelindex has in it's internalId an index in the parentCreateList.
+QList<int>
+QtGroupingProxy::addSourceRow( const QModelIndex &idx )
+{
+    QList<int> updatedGroups;
+
+    QList<RowData> groupData = belongsTo( idx );
+    //an item can be in multiple groups
+    foreach( RowData data, groupData )
+    {
+        int groupIndex = -1;
+        if( !data.isEmpty() )
+        {
+//            qDebug() << QString("index %1 belongs to group %2").arg( row )
+//                         .arg( data[0][Qt::DisplayRole].toString() );
+
+            foreach( const RowData &cachedData, m_groupMaps )
+            {
+                //when this matches the index belongs to an existing group
+                if( data[0][Qt::DisplayRole] == cachedData[0][Qt::DisplayRole] )
+                {
+                    data = cachedData;
+                    break;
+                }
+            }
+
+            groupIndex = m_groupMaps.indexOf( data );
+            //-1 means not found
+            if( groupIndex == -1 )
+            {
+                //new groups are added to the end of the existing list
+                m_groupMaps << data;
+                groupIndex = m_groupMaps.count() - 1;
+            }
+        }
+
+        if( !updatedGroups.contains( groupIndex ) )
+            updatedGroups << groupIndex;
+
+        //it's possible it's a new group, QHash will create it for us with this call:
+        m_groupHash[groupIndex];
+
+        //update the groupHash to the new source-model layout (one row added)
+        QMutableHashIterator<quint32, QList<int> > i( m_groupHash );
+        while( i.hasNext() )
+        {
+            i.next();
+            QList<int> &groupList = i.value();
+            int insertedProxyRow = groupList.count();
+            for( ; insertedProxyRow > 0 ; insertedProxyRow-- )
+            {
+                int &rowValue = groupList[insertedProxyRow-1];
+                if( idx.row() <= rowValue )
+                {
+                    //increment the rows that come after the new row since they moved one place up.
+                    rowValue++;
+                }
+                else
+                {
+                    break;
+                }
+            }
+            //the row needs to be added to this group
+            if( i.key() == (quint32)groupIndex )
+                groupList.insert( insertedProxyRow, idx.row() );
+        }
+    }
+    return updatedGroups;
+}
+
+/** Each ModelIndex has in it's internalId a position in the parentCreateList.
   * struct ParentCreate are the instructions to recreate the parent index.
-  * It contains the row number and the index in this list of the grandfather.
+  * It contains the row number and the postion in this list of the grandfather.
   * This function creates the ParentCreate structs and saves them in a list.
   */
 int
@@ -210,9 +252,9 @@ QtGroupingProxy::indexOfParentCreate( const QModelIndex &parent ) const
 }
 
 QModelIndex
-QtGroupingProxy::index( int row, int column, const QModelIndex& parent ) const
+QtGroupingProxy::index( int row, int column, const QModelIndex &parent ) const
 {
-    //qDebug() << "index requested for: (" << row << "," << column << "), " << parent;
+//    qDebug() << "index requested for: (" << row << "," << column << "), " << parent;
     if( !hasIndex(row, column, parent) )
         return QModelIndex();
 
@@ -252,7 +294,7 @@ QtGroupingProxy::rowCount( const QModelIndex& index ) const
     if( !index.isValid() )
     {
         //the number of top level groups + the number of non-grouped playlists
-        int rows = m_groupMaps.count() + m_groupHash.values( -1 ).count();
+        int rows = m_groupMaps.count() + m_groupHash.value( -1 ).count();
         //qDebug() << rows << " in root group";
         return rows;
     }
@@ -261,13 +303,13 @@ QtGroupingProxy::rowCount( const QModelIndex& index ) const
     if( isGroup( index ) )
     {
         qint64 groupIndex = index.row();
-        int rows = m_groupHash.count( groupIndex );
+        int rows = m_groupHash.value( groupIndex ).count();
         //qDebug() << rows << " in group " << m_groupMaps[groupIndex];
         return rows;
     }
 
     QModelIndex originalIndex = mapToSource( index );
-    int rowCount = m_model->rowCount( originalIndex );
+    int rowCount = sourceModel()->rowCount( originalIndex );
     //qDebug() << "original item: rowCount == " << rowCount;
     return rowCount;
 }
@@ -275,7 +317,7 @@ QtGroupingProxy::rowCount( const QModelIndex& index ) const
 int
 QtGroupingProxy::columnCount( const QModelIndex& index ) const
 {
-    return m_model->columnCount( mapToSource( index ) );
+    return sourceModel()->columnCount( mapToSource( index ) );
 }
 
 QVariant
@@ -302,7 +344,7 @@ QtGroupingProxy::data( const QModelIndex &index, int role ) const
 
         //map all data from children to columns of group to allow grouping one level up
         QVariantList variantsOfChildren;
-        int childCount = m_groupHash.count( row );
+        int childCount = m_groupHash.value( row ).count();
         if( childCount == 0 )
             return QVariant();
 
@@ -319,7 +361,7 @@ QtGroupingProxy::data( const QModelIndex &index, int role ) const
         }
         //qDebug() << "gathered this data from children: " << variantsOfChildren;
         //saving in cache
-        RoleVariantMap roleMap = m_groupMaps[row].value( column );
+        IndexData roleMap = m_groupMaps[row].value( column );
         foreach( const QVariant &variant, variantsOfChildren )
         {
             if( roleMap[ role ] != variant )
@@ -352,29 +394,31 @@ QtGroupingProxy::setData( const QModelIndex &idx, const QVariant &value, int rol
 
     if( isGroup( idx ) )
     {
-        RoleVariantMap columnData = m_groupMaps[idx.row()][idx.column()];
+        IndexData columnData = m_groupMaps[idx.row()][idx.column()];
 
         columnData.insert( role, value );
-        //also set the display role if we are changing the grouped column data or the
-        //first column
-        if( idx.column() == m_groupedColumn || idx.column() == 0 )
+        //QItemDelegate will always use Qt::EditRole
+        if( role == Qt::EditRole )
             columnData.insert( Qt::DisplayRole, value );
+
         //and make sure it's stored in the map
         m_groupMaps[idx.row()].insert( idx.column(), columnData );
 
-        int columnToChange = idx.row() != 0 ? idx.row() : m_groupedColumn;
-        foreach( int originalRow, m_groupHash.values( idx.row() ) )
+        int columnToChange = idx.column() ? idx.column() : m_groupedColumn;
+        foreach( int originalRow, m_groupHash.value( idx.row() ) )
         {
-            QModelIndex childIdx = m_model->index( originalRow, columnToChange,
+            QModelIndex childIdx = sourceModel()->index( originalRow, columnToChange,
                                                    m_rootNode );
             if( childIdx.isValid() )
-                m_model->setData( childIdx, value, role );
+                sourceModel()->setData( childIdx, value, role );
         }
+        //TODO: we might need to reload the data from the children at this point
+
         emit dataChanged( idx, idx );
         return true;
     }
 
-    return m_model->setData( mapToSource( idx ), value, role );
+    return sourceModel()->setData( mapToSource( idx ), value, role );
 }
 
 bool
@@ -391,7 +435,7 @@ QtGroupingProxy::mapToSource( const QModelIndex& index ) const
 {
     //qDebug() << "mapToSource: " << index;
     if( !index.isValid() )
-        return QModelIndex();
+        return m_rootNode;
 
     if( isGroup( index ) )
     {
@@ -410,14 +454,14 @@ QtGroupingProxy::mapToSource( const QModelIndex& index ) const
         if( !proxyParent.isValid() )
             indexInGroup -= m_groupMaps.count();
         //qDebug() << "indexInGroup" << indexInGroup;
-        QList<int> childRows = m_groupHash.values( proxyParent.row() );
+        QList<int> childRows = m_groupHash.value( proxyParent.row() );
         if( childRows.isEmpty() || indexInGroup >= childRows.count() || indexInGroup < 0 )
             return QModelIndex();
 
         originalRow = childRows.at( indexInGroup );
         //qDebug() << "originalRow: " << originalRow;
     }
-    return m_model->index( originalRow, index.column(), originalParent );
+    return sourceModel()->index( originalRow, index.column(), originalParent );
 }
 
 QModelIndexList
@@ -452,13 +496,23 @@ QtGroupingProxy::mapFromSource( const QModelIndex &idx ) const
     }
     else
     {
-        //idx is an item of the top level of the source model (child of the rootnode)
-        int groupRow = m_groupHash.key( sourceRow, -1 );
+        //idx is an item in the top level of the source model (child of the rootnode)
+        int groupRow = -1;
+        QHashIterator<quint32, QList<int> > iterator( m_groupHash );
+        while( iterator.hasNext() )
+        {
+            iterator.next();
+            if( iterator.value().contains( sourceRow ) )
+            {
+                groupRow = iterator.key();
+                break;
+            }
+        }
 
         if( groupRow != -1 ) //it's in a group, let's find the correct row.
         {
             proxyParent = this->index( groupRow, 0, QModelIndex() );
-            proxyRow = m_groupHash.values( groupRow ).indexOf( sourceRow );
+            proxyRow = m_groupHash.value( groupRow ).indexOf( sourceRow );
         }
         else
         {
@@ -466,7 +520,7 @@ QtGroupingProxy::mapFromSource( const QModelIndex &idx ) const
             // if the proxy item is not in a group it will be below the groups.
             int groupLength = m_groupMaps.count();
             //qDebug() << "groupNames length: " << groupLength;
-            int i = m_groupHash.values( -1 ).indexOf( sourceRow );
+            int i = m_groupHash.value( -1 ).indexOf( sourceRow );
             //qDebug() << "index in hash: " << i;
             proxyRow = groupLength + i;
         }
@@ -482,27 +536,28 @@ QtGroupingProxy::flags( const QModelIndex &idx ) const
 {
     //only if the grouped column has the editable flag set allow the
     //actions leading to setData on the source (edit, drop & drag)
+//    qDebug() << idx;
     if( isGroup( idx ) )
     {
-        dumpGroups();
+//        dumpGroups();
         Qt::ItemFlags defaultFlags( Qt::ItemIsEnabled | Qt::ItemIsSelectable );
         bool groupIsEditable = true;
 
         //it's possible to have empty groups
-        if( m_groupHash.count( idx.row() ) == 0 )
+        if( m_groupHash.value( idx.row() ).count() == 0 )
         {
             //check the flags of this column with the root node
-            QModelIndex originalRootNode = m_model->index( m_rootNode.row(), m_groupedColumn,
+            QModelIndex originalRootNode = sourceModel()->index( m_rootNode.row(), m_groupedColumn,
                                                            m_rootNode.parent() );
             groupIsEditable = originalRootNode.flags().testFlag( Qt::ItemIsEditable );
         }
         else
         {
-            foreach( int originalRow, m_groupHash.values( idx.row() ) )
+            foreach( int originalRow, m_groupHash.value( idx.row() ) )
             {
-                QModelIndex originalIdx = m_model->index( originalRow, m_groupedColumn,
+                QModelIndex originalIdx = sourceModel()->index( originalRow, m_groupedColumn,
                                                           m_rootNode );
-                qDebug() << "originalIdx: " << originalIdx;
+//                qDebug() << "originalIdx: " << originalIdx;
                 groupIsEditable = groupIsEditable
                                   ? originalIdx.flags().testFlag( Qt::ItemIsEditable )
                                   : false;
@@ -512,17 +567,17 @@ QtGroupingProxy::flags( const QModelIndex &idx ) const
         }
 
         if( groupIsEditable )
-            return (  defaultFlags | Qt::ItemIsEditable |
-                 Qt::ItemIsDropEnabled | Qt::ItemIsDragEnabled );
+            return (  defaultFlags | Qt::ItemIsEditable | Qt::ItemIsDropEnabled );
         return defaultFlags;
     }
 
     QModelIndex originalIdx = mapToSource( idx );
-    Qt::ItemFlags originalItemFlags = m_model->flags( originalIdx );
+    Qt::ItemFlags originalItemFlags = sourceModel()->flags( originalIdx );
 
     //check the source model to see if the grouped column is editable;
-    QModelIndex groupedColumnIndex = m_model->index( originalIdx.row(), m_groupedColumn, originalIdx.parent() );
-    bool groupIsEditable = m_model->flags( groupedColumnIndex ).testFlag( Qt::ItemIsEditable );
+    QModelIndex groupedColumnIndex =
+            sourceModel()->index( originalIdx.row(), m_groupedColumn, originalIdx.parent() );
+    bool groupIsEditable = sourceModel()->flags( groupedColumnIndex ).testFlag( Qt::ItemIsEditable );
     if( groupIsEditable )
         return originalItemFlags | Qt::ItemIsDragEnabled;
 
@@ -532,7 +587,7 @@ QtGroupingProxy::flags( const QModelIndex &idx ) const
 QVariant
 QtGroupingProxy::headerData( int section, Qt::Orientation orientation, int role ) const
 {
-    return m_model->headerData( section, orientation, role );
+    return sourceModel()->headerData( section, orientation, role );
 }
 
 bool
@@ -544,7 +599,7 @@ QtGroupingProxy::canFetchMore( const QModelIndex &parent ) const
     if( isGroup( parent ) )
         return false;
 
-    return m_model->canFetchMore( mapToSource( parent ) );
+    return sourceModel()->canFetchMore( mapToSource( parent ) );
 }
 
 void
@@ -556,17 +611,16 @@ QtGroupingProxy::fetchMore ( const QModelIndex & parent )
     if( isGroup( parent ) )
         return;
 
-    return m_model->fetchMore( mapToSource( parent ) );
+    return sourceModel()->fetchMore( mapToSource( parent ) );
 }
 
 QModelIndex
-QtGroupingProxy::addEmptyGroup( const ColumnVariantMap &data )
+QtGroupingProxy::addEmptyGroup( const RowData &data )
 {
     int newRow = m_groupMaps.count();
     beginInsertRows( QModelIndex(), newRow, newRow );
     m_groupMaps << data;
     endInsertRows();
-    emit layoutChanged();
     return index( newRow, 0, QModelIndex() );
 }
 
@@ -591,9 +645,9 @@ QtGroupingProxy::hasChildren( const QModelIndex &parent ) const
         return true;
 
     if( isGroup( parent ) )
-        return !m_groupHash.values( parent.row() ).isEmpty();
+        return !m_groupHash.value( parent.row() ).isEmpty();
 
-    return m_model->hasChildren( mapToSource( parent ) );
+    return sourceModel()->hasChildren( mapToSource( parent ) );
 }
 
 void
@@ -607,47 +661,55 @@ QtGroupingProxy::modelDataChanged( const QModelIndex& start, const QModelIndex&
 }
 
 void
-QtGroupingProxy::modelRowsInserted( const QModelIndex& index, int start, int end )
+QtGroupingProxy::modelRowsInserted( const QModelIndex &parent, int start, int end )
 {
-    Q_UNUSED( index )
-    Q_UNUSED( start )
-    Q_UNUSED( end )
-    //TODO: see if new groups have to be created, deleted or adjusted. Try to avoid buildTree()
-    buildTree();
-}
-
-void
-QtGroupingProxy::modelRowsRemoved( const QModelIndex& index, int start, int end )
-{
-    Q_UNUSED( index )
-    Q_UNUSED( start )
-    Q_UNUSED( end )
-    //don't call buildTree() because it will mess up with removing multiple rows
-    //individually using removeRow().
-    //TODO: this breaks the view when rows are not deleted manually.
-}
-
-void
-QtGroupingProxy::slotDeleteGroup()
-{
-}
-
-void
-QtGroupingProxy::slotRenameGroup()
-{
-    //get the name for this new group
-    const QString newName = QInputDialog::getText( 0, tr("New name"),
-                tr("Enter a new name for a group that already exists", "Enter new group name:") );
-
-    foreach( int originalRow, m_groupHash.values( m_selectedGroups.first().row() ) )
+    if( parent == m_rootNode )
     {
-        QModelIndex index = m_model->index( originalRow, 0, QModelIndex() );
-        QStringList groups;
-        groups << newName;
-        //qDebug() << "Applying " << groups << " to " << index;
+        //top level of the model changed, these new rows need to be put in groups
+        for( int modelRow = start; modelRow <= end ; modelRow++ )
+        {
+            QList<int> updatedGroups;
+            updatedGroups << addSourceRow( sourceModel()->index( modelRow, m_groupedColumn, m_rootNode ) );
+            foreach( int groupIndex, updatedGroups )
+            {
+                QModelIndex updatedProxyParent = ( groupIndex == -1 ) ? QModelIndex()
+                    : index( groupIndex, 0 );
+                int insertedProxyRow = m_groupHash[groupIndex].indexOf( modelRow );
+                //non-grouped items are below the groups, so offset the row
+                if( groupIndex == -1 )
+                    insertedProxyRow += m_groupMaps.count();
+
+                beginInsertRows( updatedProxyParent, insertedProxyRow, insertedProxyRow );
+                endInsertRows();
+            }
+        }
     }
-    buildTree();
-    emit layoutChanged();
+    else
+    {
+        //a item was added to one of the original index, remap and pass it on
+        QModelIndex proxyParent = mapFromSource( parent );
+//        qDebug() << proxyParent;
+        //TODO: move this to slot connected to rowsAboutToBeInserted()
+        beginInsertRows( proxyParent, start, end );
+        endInsertRows();
+    }
+}
+
+void
+QtGroupingProxy::modelRowsRemoved( const QModelIndex &parent, int start, int end )
+{
+    if( parent == m_rootNode )
+    {
+        //rebuild tree, reseting the view
+        //TODO: don't rebuild but remove from
+        buildTree();
+        return;
+    }
+
+    QModelIndex proxyParent = mapFromSource( parent );
+    //TODO: move this to slot connected to rowsAboutToBeRemoved()
+    beginRemoveRows( proxyParent, start, end );
+    endRemoveRows();
 }
 
 bool
@@ -664,16 +726,14 @@ QtGroupingProxy::isAGroupSelected( const QModelIndexList& list ) const
 void
 QtGroupingProxy::dumpGroups() const
 {
-    return; //so spammy
     qDebug() << "m_groupHash: ";
     for( int groupIndex = -1; groupIndex < m_groupHash.keys().count() - 1; groupIndex++ )
     {
-        qDebug() << groupIndex << " : " << m_groupHash.values( groupIndex );
+        qDebug() << groupIndex << " : " << m_groupHash.value( groupIndex );
     }
 
     qDebug() << "m_groupMaps: ";
     for( int groupIndex = 0; groupIndex < m_groupMaps.count(); groupIndex++ )
-        qDebug() << m_groupMaps[groupIndex] << ": " << m_groupHash.values( groupIndex );
-    qDebug() << m_groupHash.values( -1 );
+        qDebug() << m_groupMaps[groupIndex] << ": " << m_groupHash.value( groupIndex );
+    qDebug() << m_groupHash.value( -1 );
 }
-//#include "GroupingProxy.moc"
