@@ -16,14 +16,13 @@
 
 #include "ConfigDialog.h"
 
-#include "Amarok.h"
+#include "core/support/Amarok.h"
 #include "amarokconfig.h"
-#include "Debug.h"
+#include "core/support/Debug.h"
 
 #include "CollectionConfig.h"
 #include "DatabaseConfig.h"
 #include "GeneralConfig.h"
-#include "MainWindow.h"
 //#include "MediadeviceConfig.h"
 #include "NotificationsConfig.h"
 #include "PlaybackConfig.h"
