@@ -15,16 +15,17 @@
  * this program.  If not, see <http://www.gnu.org/licenses/>.                           *
  ****************************************************************************************/
 
+#define DEBUG_PREFIX "Context::Applet"
+
 #include "Applet.h"
 
-#include "App.h"
+#include "AppletHeader.h"
+#include "amarokconfig.h"
 #include "Containment.h"
 #include "ContextView.h"
-#include "Debug.h"
+#include "core/support/Debug.h"
 #include "PaletteHandler.h"
 
-#include <Plasma/Animator>
-#include <Plasma/FrameSvg>
 #include <Plasma/IconWidget>
 #include <Plasma/Theme>
 
@@ -32,6 +33,8 @@
 #include <QGraphicsScene>
 #include <QFontMetrics>
 #include <QPainter>
+#include <QPropertyAnimation>
+
 #include <KServiceTypeTrader>
 
 namespace Context
@@ -41,32 +44,21 @@ namespace Context
 
 Context::Applet::Applet( QObject * parent, const QVariantList& args )
     : Plasma::Applet( parent, args )
-    , m_collapsed( false )
-    , m_animationIdOn( 0 )
-    , m_animationIdOff( 0 )
+    , m_canAnimate( !KServiceTypeTrader::self()->query("Plasma/Animator", QString()).isEmpty() )
+    , m_heightCollapseOff( 0 )
+    , m_header( 0 )
     , m_transient( 0 )
+    , m_isMessageShown( false )
     , m_standardPadding( 6.0 )
-    , m_textBackground( 0 )
 {
-    KService::List offers = KServiceTypeTrader::self()->query("Plasma/Animator", QString());
-    m_canAnimate = !offers.isEmpty();
-
-    if( m_canAnimate )
-        connect ( Plasma::Animator::self(), SIGNAL(customAnimationFinished ( int ) ), this, SLOT( animateEnd( int ) ) );
-    setBackgroundHints(NoBackground);
-
-    connect( The::paletteHandler(), SIGNAL( newPalette( const QPalette& ) ), SLOT(  paletteChanged( const QPalette &  ) ) );
+    setBackgroundHints( NoBackground );
+    connect( The::paletteHandler(), SIGNAL(newPalette(QPalette)), SLOT(paletteChanged(QPalette )) );
 }
 
-Context::Applet::~Applet( )
+Context::Applet::~Applet()
 {
-    if ( m_animationIdOn != 0 )
-        Plasma::Animator::self()->stopCustomAnimation( m_animationIdOn );
-    if ( m_animationIdOff != 0 )
-        Plasma::Animator::self()->stopCustomAnimation( m_animationIdOff );
 }
 
-
 QFont
 Context::Applet::shrinkTextSizeToFit( const QString& text, const QRectF& bounds )
 {
@@ -100,65 +92,44 @@ Context::Applet::shrinkTextSizeToFit( const QString& text, const QRectF& bounds
 }
 
 QString
-Context::Applet::truncateTextToFit( QString text, const QFont& font, const QRectF& bounds )
+Context::Applet::truncateTextToFit( const QString &text, const QFont& font, const QRectF& bounds )
 {
     QFontMetrics fm( font );
     return fm.elidedText ( text, Qt::ElideRight, (int)bounds.width() );
 }
 
 void
-Context::Applet::drawRoundedRectAroundText( QPainter* p, QGraphicsTextItem* textItem )
+Context::Applet::paintInterface( QPainter *p,
+                                 const QStyleOptionGraphicsItem *option,
+                                 const QRect &contentsRect )
 {
-    p->save();
-    p->setRenderHint( QPainter::Antialiasing );
-
-    if ( !m_textBackground )
-    {
-        m_textBackground = new Plasma::FrameSvg();
-        m_textBackground->setImagePath( "widgets/text-background" );
-        m_textBackground->setEnabledBorders( Plasma::FrameSvg::AllBorders );
-    }
-
-    // Paint in integer coordinates, align to grid
-    QRectF rect = textItem->boundingRect();
-    rect.setX( qRound( rect.x() ) );
-    rect.setY( qRound( rect.y() ) );
-    rect.setHeight( qRound( rect.height() ) );
-    rect.setWidth( qRound( rect.width() ) );
-    rect.moveTopLeft( textItem->pos() );
-
-    QPointF pos = textItem->pos();
-    pos.setX( qRound( pos.x() ) );
-    pos.setY( qRound( pos.y() ) );
-
-    rect.moveTopLeft( pos );
-    rect.adjust( -5, -5, 5, 5 );
-
-    m_textBackground->resizeFrame( rect.size() );
-    m_textBackground->paintFrame( p, rect.topLeft() );
-    p->restore();
+    Plasma::Applet::paintInterface( p, option, contentsRect );
+    addGradientToAppletBackground( p );
 }
 
 void
 Context::Applet::addGradientToAppletBackground( QPainter* p )
 {
-        // tint the whole applet
+    // tint the whole applet
     // draw non-gradient backround. going for elegance and style
+    const QRectF roundRect = boundingRect().adjusted( 0, 1, -1, -1 );
+
     p->save();
+    p->setRenderHint( QPainter::Antialiasing );
     QPainterPath path;
-    path.addRoundedRect( boundingRect().adjusted( 0, 1, -1, -1 ), 4, 4 );
-    //p->fillPath( path, gradient );
+    path.addRoundedRect( roundRect, 4, 4 );
     QColor highlight = PaletteHandler::highlightColor( 0.4, 1.05 );
     highlight.setAlphaF( highlight.alphaF() * 0.5 );
     p->fillPath( path, highlight );
     p->restore();
 
     p->save();
+    p->setRenderHint( QPainter::Antialiasing );
     p->translate( 0.5, 0.5 );
     QColor col = PaletteHandler::highlightColor( 0.3, 0.5 );
     col.setAlphaF( col.alphaF() * 0.7 );
     p->setPen( col );
-    p->drawRoundedRect( boundingRect().adjusted( 0, 1, -1, -1 ), 4, 4 );
+    p->drawRoundedRect( roundRect, 4, 4 );
     p->restore();
 }
 
@@ -206,27 +177,13 @@ Context::Applet::cleanUpAndDelete()
     Plasma::Applet::deleteLater();
 }
 
-QSizeF
-Context::Applet::sizeHint( Qt::SizeHint which, const QSizeF & constraint ) const
-{
-    return QSizeF( QGraphicsWidget::sizeHint( which, constraint ).width(), m_heightCurrent );
-}
-
-void
-Context::Applet::resize( qreal wid, qreal hei)
-{
-    m_heightCollapseOff = hei;
-    m_heightCurrent = hei;
-    Plasma::Applet::resize( wid, hei );
-}
-
 Plasma::IconWidget*
-Context::Applet::addAction( QAction *action, const int size )
+Context::Applet::addAction( QGraphicsItem *parent, QAction *action, const int size )
 {
     if( !action )
         return 0;
 
-    Plasma::IconWidget *tool = new Plasma::IconWidget( this );
+    Plasma::IconWidget *tool = new Plasma::IconWidget( parent );
     tool->setAction( action );
     tool->setText( QString() );
     tool->setToolTip( action->text() );
@@ -241,81 +198,141 @@ Context::Applet::addAction( QAction *action, const int size )
     return tool;
 }
 
+void
+Context::Applet::enableHeader( bool enable )
+{
+    if( m_header )
+    {
+        delete m_header;
+        m_header = 0;
+    }
+
+    if( enable )
+    {
+        m_header = new AppletHeader( this );
+        m_header->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed );
+    }
+}
+
+Plasma::IconWidget *
+Context::Applet::addLeftHeaderAction( QAction *action )
+{
+    if( !m_header )
+        return 0;
+
+    Plasma::IconWidget *icon = addAction( 0, action );
+    m_header->addIcon( icon, Qt::AlignLeft );
+    return icon;
+}
+
+Plasma::IconWidget *
+Context::Applet::addRightHeaderAction( QAction *action )
+{
+    if( !m_header )
+        return 0;
+
+    Plasma::IconWidget *icon = addAction( 0, action );
+    m_header->addIcon( icon, Qt::AlignRight );
+    return icon;
+}
+
+QString
+Context::Applet::headerText() const
+{
+    if( !m_header )
+        return QString();
+    return m_header->titleText();
+}
+
+void
+Context::Applet::setHeaderText( const QString &text )
+{
+    if( !m_header )
+        return;
+    m_header->setTitleText( text );
+}
+
+bool
+Context::Applet::isAnimating() const
+{
+    QSharedPointer<QPropertyAnimation> anim = m_animation.toStrongRef();
+    if( anim )
+        return (anim->state() == QAbstractAnimation::Running);
+    return false;
+}
+
+bool
+Context::Applet::isCollapsed() const
+{
+    return m_heightCollapseOn == preferredHeight();
+}
+
 void
 Context::Applet::setCollapseOn()
 {
-
-    if( !m_canAnimate ) {
-        resize( size().width(), m_heightCollapseOn );
-        return;
-    }
-
-    // We are asked to collapse, but the applet is already animating to collapse, do nothing
-    if ( m_animationIdOn != 0 )
-        return;
-
-    // We are already collapsed
-    if ( size().height() == m_heightCollapseOn )
-        return;
-
-    debug() << "collapsing applet to..." << m_heightCollapseOn;
-    if( m_heightCollapseOff == -1 )
-        m_animFromHeight = size().height();
-    else
-        m_animFromHeight = m_heightCollapseOff;
-
-    if ( m_animationIdOff != 0 ) // warning we are extanding right now we should stop
-    {
-        // stop the anim
-        Plasma::Animator::self()->stopCustomAnimation( m_animationIdOff );
-        m_animationIdOff = 0;
-    }
-    m_collapsed = false;
-    m_animationIdOn = Plasma::Animator::self()->customAnimation(20, 1000, Plasma::Animator::EaseInCurve, this, "animateOn" );
+    collapse( true );
 }
 
 void
 Context::Applet::setCollapseOff()
 {
-    DEBUG_BLOCK
+    collapse( false );
+}
 
-    if( !m_canAnimate ) {
-        resize( size().width(), m_heightCollapseOff );
-        return;
-    }
-    
-    // We are asked to extend, but the applet is already animating to extend, do nothing
-    if ( m_animationIdOff != 0 )
-        return;
-    
-    // Applet already extended
-    if ( size().height() == m_heightCollapseOff )
+void
+Context::Applet::collapse( bool on )
+{
+    qreal finalHeight = ( on ) ? m_heightCollapseOn : m_heightCollapseOff;
+    const qreal maxHeight = containment()->size().height();
+    if( (finalHeight > maxHeight) || (finalHeight < 0) )
+        finalHeight = maxHeight;
+
+    if( finalHeight == size().height() )
         return;
 
-    debug() << "height:" << size().height() << "target:" << m_heightCollapseOff << "m_collapsed:" << m_collapsed;
-    if( m_heightCollapseOff == -1) // if it's self-expanding, don't animate as we don't know where we're going. also, if we're shrinking
-    {                                                                       // stop that and expand regardless
-        // stop the animation on now
-        if( m_animationIdOn != 0 )
-        {
-            Plasma::Animator::self()->stopCustomAnimation( m_animationIdOn );
-            m_animationIdOn = 0;
-        }
-        m_heightCurrent =  m_heightCollapseOff;
-        emit sizeHintChanged(Qt::PreferredSize);
-        updateGeometry();
-        m_collapsed = false;
-        return;
-    }
-
-    if ( m_animationIdOn != 0 ) // warning we are collapsing right now, stop that and reverse it !
+    if( !AmarokConfig::animateAppletCollapse() )
     {
-        // stop the anim
-        Plasma::Animator::self()->stopCustomAnimation( m_animationIdOn );
-        m_animationIdOn = 0;
+        setPreferredHeight( finalHeight );
+        emit sizeHintChanged( Qt::PreferredSize );
+        updateGeometry();
+        return;
     }
-    m_collapsed = true ;
-    m_animationIdOff = Plasma::Animator::self()->customAnimation(20, 1000, Plasma::Animator::EaseInCurve, this, "animateOff" );
+
+    // debug() << pluginName() << (on ? "collapsing to" : "uncollapsing to") << finalHeight;
+    QPropertyAnimation *pan = m_animation.data();
+    if( !pan )
+        pan = new QPropertyAnimation( this, "preferredSize" );
+    if( pan->state() == QAbstractAnimation::Running )
+        pan->stop();
+    pan->setDuration( 600 );
+    pan->setEasingCurve( QEasingCurve::InQuad );
+    pan->setStartValue( size() );
+    pan->setEndValue( QSizeF(size().width(), finalHeight) );
+    connect( pan, SIGNAL(finished()), SLOT(collapseAnimationFinished()) );
+    m_animation = pan;
+    pan->setDirection( QAbstractAnimation::Forward );
+    ContextView *v = static_cast<ContextView*>( view() );
+    v->addCollapseAnimation( pan );
+}
+
+int
+Context::Applet::collapseHeight() const
+{
+    return m_heightCollapseOn;
+}
+
+int
+Context::Applet::collapseOffHeight() const
+{
+    return m_heightCollapseOff;
+}
+
+void
+Context::Applet::collapseAnimationFinished()
+{
+    emit sizeHintChanged( Qt::PreferredSize );
+    updateConstraints();
+    update();
 }
 
 void
@@ -324,50 +341,10 @@ Context::Applet::setCollapseHeight( int h )
     m_heightCollapseOn = h;
 }
 
-bool
-Context::Applet::isAppletCollapsed()
-{
-    return ( m_heightCollapseOn == m_heightCurrent );
-}
-
-bool
-Context::Applet::isAppletExtended()
-{
-    return ( m_heightCollapseOff == m_heightCurrent );
-}
-
-
 void
-Context::Applet::animateOn( qreal anim )
+Context::Applet::setCollapseOffHeight( int h )
 {
-    m_heightCurrent = m_animFromHeight - ( m_animFromHeight - m_heightCollapseOn ) * anim ;
-    emit sizeHintChanged(Qt::PreferredSize);
-}
-
-void
-Context::Applet::animateOff( qreal anim )
-{
-    m_heightCurrent =  m_heightCollapseOn + ( m_heightCollapseOff - m_heightCollapseOn ) * anim ;
-    emit sizeHintChanged(Qt::PreferredSize);
-}
-
-void
-Context::Applet::animateEnd( int id )
-{
-    if( id == m_animationIdOn )
-    {
-        Plasma::Applet::resize( size().width(), m_heightCollapseOn );
-        m_collapsed = true;
-        m_animationIdOn = 0;
-    }
-    if ( id == m_animationIdOff )
-    {
-        Plasma::Applet::resize( size().width(), m_heightCollapseOff );
-        m_collapsed = false;
-        m_animationIdOff = 0;
-    }
-    updateGeometry();
-    emit sizeHintChanged(Qt::PreferredSize);
+    m_heightCollapseOff = h;
 }
 
 void
@@ -376,9 +353,50 @@ Context::Applet::paletteChanged( const QPalette & palette )
     Q_UNUSED( palette )
 }
 
+void
+Context::Applet::plasmaMessageHidden()
+{
+    // Disconnect from the messageButtonPressed() signal so the next
+    // dialog can be shown.
+    disconnect( SIGNAL( messageButtonPressed( MessageButton ) ) );
+
+    // No dialog is shown anymore.
+    m_isMessageShown = false;
+}
+
 bool Context::Applet::canAnimate()
 {
     return m_canAnimate;
 }
 
+void
+Context::Applet::showWarning( const QString &message, const char *slot )
+{
+    // Show a message with the "warning" icon.
+    showMessage( message, slot, KIcon( "dialog-warning" ) );
+}
+
+void
+Context::Applet::showMessage( const QString &message, const char *slot, const KIcon &icon )
+{
+    DEBUG_BLOCK
+
+    // Only show the message if none is shown yet.
+    if( !m_isMessageShown )
+    {
+        // Make sure no one else can show a dialog.
+        m_isMessageShown = true;
+
+        // Get the "Yes" and "No" buttons.
+        Plasma::MessageButtons plasmaYesNoButtons = Plasma::ButtonYes | Plasma::ButtonNo;
+
+        // Connect Plasma's messageButtonPressed SIGNAL to the given slot.
+        connect( this, SIGNAL( messageButtonPressed( MessageButton ) ), slot );
+        connect( this, SIGNAL( messageButtonPressed( MessageButton ) ), SLOT( plasmaMessageHidden() ) );
+
+        // Show a dialog and ask the user what to do.
+        Plasma::Applet::showMessage( icon, message, plasmaYesNoButtons );
+    }
+}
+
 #include "Applet.moc"
