@@ -14,10 +14,12 @@
  * this program.  If not, see <http://www.gnu.org/licenses/>.                           *
  ****************************************************************************************/
 
+#define DEBUG_PREFIX "VerticalToolbarContainment"
+
 #include "VerticalToolbarContainment.h"
 
 #include "ContextView.h"
-#include "Debug.h"
+#include "core/support/Debug.h"
 #include "PaletteHandler.h"
 #include "VerticalAppletLayout.h"
 
@@ -25,13 +27,10 @@
 
 #include <QGraphicsLinearLayout>
 
-#define TOOLBAR_X_OFFSET 2000
-
-
 Context::VerticalToolbarContainment::VerticalToolbarContainment( QObject *parent, const QVariantList &args )
     : Containment( parent, args )
+    , m_view( 0 )
     , m_applets( 0 )
-    , m_noApplets( true )
     , m_noAppletText( 0 )
 {
     DEBUG_BLOCK
@@ -43,19 +42,12 @@ Context::VerticalToolbarContainment::VerticalToolbarContainment( QObject *parent
     debug() << "applet containment has corona:" << corona();
     m_applets = new VerticalAppletLayout( this );
 
-    connect( this, SIGNAL( appletRemoved( Plasma::Applet* ) ), SLOT( appletRemoved( Plasma::Applet* ) ) );
-    connect( this, SIGNAL( appletRemoved( Plasma::Applet* ) ), SIGNAL( geometryChanged() ) );
+    connect( this, SIGNAL(appletRemoved(Plasma::Applet*)), SLOT(appletRemoved(Plasma::Applet*)) );
+    connect( this, SIGNAL(appletRemoved(Plasma::Applet*)), SIGNAL(geometryChanged()) );
 
-    connect( m_applets,  SIGNAL( appletAdded( Plasma::Applet*, int ) ), SIGNAL( appletAdded( Plasma::Applet*, int) ) ); // goes out to applet toolbar
-    connect( m_applets, SIGNAL(  appletAdded( Plasma::Applet*, int ) ), SIGNAL( geometryChanged() ) );
-
-    connect( m_applets, SIGNAL( noApplets( bool ) ), SLOT( showEmptyText( bool ) ) );
-
-    m_noAppletText = new QGraphicsTextItem( this );
-    m_noAppletText->setHtml( QString( "<html>  <style type=\"text/css\"> body { background-color: %1; } </style> \
-                                        <body> <p align=\"center\"> %2 </</p></body></html>" )
-                                       .arg( The::paletteHandler()->highlightColor().name() )
-                                       .arg( i18n( "Please add some applets from the toolbar at the bottom of the context view." ) ) );
+    connect( m_applets, SIGNAL(appletAdded(Plasma::Applet*, int)), SIGNAL(appletAdded(Plasma::Applet*, int)) );
+    connect( m_applets, SIGNAL(appletAdded(Plasma::Applet*, int)), SIGNAL(geometryChanged()) );
+    connect( m_applets, SIGNAL(noApplets(bool)), SLOT(showEmptyText(bool)) );
 
 }
 
@@ -66,14 +58,13 @@ void
 Context::VerticalToolbarContainment::constraintsEvent( Plasma::Constraints constraints )
 {
     Q_UNUSED( constraints )
-
-    debug() << "setting applets geom to" << contentsRect();
-    m_applets->setGeometry( contentsRect() );
-    
-    QRectF masterRect = contentsRect();
-    m_noAppletText->setTextWidth( masterRect.width() * .4 );
-    QPointF topLeft( ( masterRect.width() / 2 ) - ( m_noAppletText->boundingRect().width() / 2 ), ( masterRect.height() / 2 ) - ( m_noAppletText->boundingRect().height() / 2 ) );
-    m_noAppletText->setPos( topLeft );
+    if( m_noAppletText )
+    {
+        QRectF masterRect = contentsRect();
+        m_noAppletText->setTextWidth( masterRect.width() * .4 );
+        QPointF topLeft( ( masterRect.width() / 2 ) - ( m_noAppletText->boundingRect().width() / 2 ), ( masterRect.height() / 2 ) - ( m_noAppletText->boundingRect().height() / 2 ) );
+        m_noAppletText->setPos( topLeft );
+    }
 }
 
 QList<QAction*>
@@ -82,14 +73,6 @@ Context::VerticalToolbarContainment::contextualActions()
     return QList< QAction* >();
 }
 
-void
-Context::VerticalToolbarContainment::paintInterface(QPainter *painter, const QStyleOptionGraphicsItem *option, const QRect& contentsRect)
-{
-    Q_UNUSED( painter );
-    Q_UNUSED( option );
-    Q_UNUSED( contentsRect );
-}
-
 void
 Context::VerticalToolbarContainment::saveToConfig( KConfigGroup &conf )
 {
@@ -106,6 +89,7 @@ Context::VerticalToolbarContainment::loadConfig( const KConfigGroup &conf )
 
     foreach( const QString& plugin, plugins )
     {
+        PERF_LOG( qPrintable( QString("Adding applet: %1").arg( plugin ) ) )
         debug() << "Adding applet: " << plugin;
         addApplet( plugin, -1 );
     }
@@ -130,10 +114,14 @@ Context::VerticalToolbarContainment::view()
     return m_view;
 }
 
-QRectF
-Context::VerticalToolbarContainment::boundingRect () const
+void
+Context::VerticalToolbarContainment::updateGeometry()
 {
-    return QRectF( QPointF( 0, 0), m_applets->totalSize() );
+    Context::Containment::updateGeometry();
+    QRectF rect = scene()->sceneRect();
+    setGeometry( rect );
+    m_applets->setGeometry( rect );
+    m_applets->refresh();
 }
 
 Plasma::Applet*
@@ -148,6 +136,8 @@ Context::VerticalToolbarContainment::addApplet( const QString& pluginName, const
     else
         m_applets->addApplet( applet, loc );
 
+     applet->setFlag(QGraphicsItem::ItemIsMovable, false);
+
     return applet;
 }
 
@@ -173,18 +163,28 @@ void
 Context::VerticalToolbarContainment::wheelEvent( QGraphicsSceneWheelEvent* event )
 {
     Q_UNUSED( event )
-
     //eat wheel events, we don't want scrolling
 }
 
 void
 Context::VerticalToolbarContainment::showEmptyText( bool toShow ) // SLOT
 {
-    m_noApplets = toShow;
     if( toShow )
+    {
+        if( !m_noAppletText )
+        {
+            m_noAppletText = new QGraphicsTextItem( this );
+            m_noAppletText->setHtml( QString( "<html>  <style type=\"text/css\"> body { background-color: %1; } </style> \
+                                              <body> <p align=\"center\"> %2 </</p></body></html>" )
+                                     .arg( The::paletteHandler()->highlightColor().name() )
+                                     .arg( i18n( "Please add some applets from the toolbar at the bottom of the context view." ) ) );
+        }
         m_noAppletText->show();
-    else
+    }
+    else if( m_noAppletText )
+    {
         m_noAppletText->hide();
+    }
     updateConstraints();
     update();
 }
