@@ -24,9 +24,9 @@
 
 #include "ScriptManager.h"
 
-#include "Amarok.h"
+#include "core/support/Amarok.h"
 #include "AmarokProcess.h"
-#include "Debug.h"
+#include "core/support/Debug.h"
 #include "EngineController.h"
 #include "MainWindow.h"
 #include "Osd.h"
@@ -87,7 +87,6 @@ ScriptManager* ScriptManager::s_instance = 0;
 
 ScriptManager::ScriptManager( QWidget* parent )
         : KDialog( parent )
-        , EngineObserver( The::engineController() )
 {
     DEBUG_BLOCK
     Ui::ScriptManagerBase gui;
@@ -232,6 +231,7 @@ ScriptManager::notifyFetchLyrics( const QString& artist, const QString& title )
 void
 ScriptManager::notifyFetchLyricsByUrl( const QString& artist, const QString& title, const QString& url )
 {
+    DEBUG_BLOCK
     emit fetchLyrics( Qt::escape( artist ), Qt::escape( title ), url );
 }
 
@@ -476,7 +476,11 @@ ScriptManager::slotRunScript( QString name, bool silent )
     m_scripts[name].running = true;
     m_scripts[name].evaluating = true;
     if( m_scripts[name].info.category() == "Lyrics" )
+    {
         m_lyricsScript = name;
+        debug() << "lyrics script started:" << name;
+        emit lyricsScriptStarted();
+    }
 
     m_scripts[name].log += time.currentTime().toString() + " Script Started!" + '\n';
     m_scripts[name].engine->setProcessEventsInterval( 100 );
@@ -615,6 +619,7 @@ ScriptManager::showScriptStalledDialog()  // SLOT
                                                            KStandardGuiItem::no(),
                                                            QString(),
                                                            KMessageBox::Dangerous );
+    Q_UNUSED(reply)
 }
 
 
@@ -690,9 +695,9 @@ ScriptManager::findScripts()
         else if ( m_scripts[key].info.category() == "Scriptable Service" )
             ServiceInfoList.append( m_scripts[key].info );
     }
-    m_scriptSelector->addScripts( GenericInfoList, KPluginSelector::ReadConfigFile, "Generic" );
-    m_scriptSelector->addScripts( LyricsInfoList, KPluginSelector::ReadConfigFile, "Lyrics" );
-    m_scriptSelector->addScripts( ServiceInfoList, KPluginSelector::ReadConfigFile, "Scriptable Service" );
+    m_scriptSelector->addScripts( GenericInfoList, KPluginSelector::ReadConfigFile, i18n( "Generic" ) );
+    m_scriptSelector->addScripts( LyricsInfoList, KPluginSelector::ReadConfigFile, i18n( "Lyrics" ) );
+    m_scriptSelector->addScripts( ServiceInfoList, KPluginSelector::ReadConfigFile, i18n( "Scriptable Service" ) );
 
     // Handle auto-run:
     slotConfigChanged( true );
