@@ -2,7 +2,7 @@
  * Copyright (c) 2004 Mark Kretschmann <kretschmann@kde.org>                            *
  * Copyright (c) 2004 Pierpaolo Di Panfilo <pippo_dp@libero.it>                         *
  * Copyright (c) 2005-2006 Alexandre Pereira de Oliveira <aleprj@gmail.com>             *
- * Copyright (c) 2008 Teo Mrnjavac <teo.mrnjavac@gmail.com>                             *
+ * Copyright (c) 2008 Téo Mrnjavac <teo@kde.org>                                        *
  * Copyright (c) 2008 Leo Franchi <lfranchi@kde.org>                                    *
  * Copyright (c) 2009 Daniel Dewald <Daniel.Dewald@time-shift.de>                       *
  * Copyright (c) 2009 Pierre Dumuid <pmdumuid@gmail.com>                                *
@@ -24,25 +24,27 @@
 
 #include "TagDialog.h"
 
-#include "Amarok.h"
+#include "core/support/Amarok.h"
+#include "core/support/Components.h"
 #include "amarokconfig.h"
-#include "CollectionManager.h"
+#include "core-impl/collections/support/CollectionManager.h"
 #include "CoverLabel.h"
-#include "SqlStorage.h"
+#include "core/collections/support/SqlStorage.h"
 #include "covermanager/CoverFetchingActions.h"
-#include "Debug.h"
-#include "EditCapability.h"
+#include "core/support/Debug.h"
+#include "core/capabilities/EditCapability.h"
 #include "FilenameLayoutDialog.h"
 #include "MainWindow.h"
-#include "MetaQueryMaker.h"
-#include "MetaUtility.h"
-#include "meta/capabilities/ReadLabelCapability.h"
-#include "meta/capabilities/WriteLabelCapability.h"
-#include "QueryMaker.h"
-#include "statusbar/StatusBar.h"       //for status messages
+#include "MusicBrainzTagger.h"
+#include "core/collections/MetaQueryMaker.h"
+#include "core/meta/support/MetaUtility.h"
+#include "core/capabilities/ReadLabelCapability.h"
+#include "core/capabilities/WriteLabelCapability.h"
+#include "core/collections/QueryMaker.h"
 #include "TagGuesser.h"
 #include "ui_TagDialogBase.h"
-#include "UpdateCapability.h"
+#include "core/capabilities/UpdateCapability.h"
+#include "core/interfaces/Logger.h"
 
 #include <KGlobal>
 #include <KHTMLView>
@@ -75,25 +77,22 @@ TagDialog::TagDialog( const Meta::TrackList &tracks, QWidget *parent )
 TagDialog::TagDialog( Meta::TrackPtr track, QWidget *parent )
     : KDialog( parent )
     , m_currentCover()
-    , m_tracks()
+    , m_tracks( Meta::TrackList() << track )
     , m_trackIterator( m_tracks )   //makes the compiler happy
     , m_queryMaker( 0 )
     , ui( new Ui::TagDialogBase() )
 {
     DEBUG_BLOCK
 
-    setCurrentTrack( m_tracks.first() );
-
-    m_tracks.append( track );
-    //we changed the list after creating the iterator, so create a new iterator
-    m_trackIterator = QListIterator< Meta::TrackPtr >( m_tracks );
+    setCurrentTrack( track );
     ui->setupUi( mainWidget() );
     resize( minimumSizeHint() );
     init();
     startDataQuery();
+    QTimer::singleShot( 0, this, SLOT(show()) );
 }
 
-TagDialog::TagDialog( QueryMaker *qm )
+TagDialog::TagDialog( Collections::QueryMaker *qm )
     : KDialog( The::mainWindow() )
     , m_currentCover()
     , m_tracks()
@@ -107,7 +106,7 @@ TagDialog::TagDialog( QueryMaker *qm )
     ui->setupUi( mainWidget() );
     resize( minimumSizeHint() );
     startDataQuery();
-    qm->setQueryType( QueryMaker::Track );
+    qm->setQueryType( Collections::QueryMaker::Track );
     connect( qm, SIGNAL( newResultReady( QString, Meta::TrackList ) ), this, SLOT( resultReady( QString, Meta::TrackList ) ), Qt::QueuedConnection );
     connect( qm, SIGNAL( queryDone() ), this, SLOT( queryDone() ), Qt::QueuedConnection );
     qm->run();
@@ -186,9 +185,10 @@ TagDialog::resultReady( const QString &collectionId, const Meta::AlbumList &albu
     {
         if( !album->name().isEmpty() )
             m_albums << album->name();
-    }
 
-    m_albums.sort();
+        if( album->hasAlbumArtist() && !album->albumArtist()->name().isEmpty() )
+            m_albumArtists << album->albumArtist()->name();
+    }
 }
 
 void
@@ -201,8 +201,6 @@ TagDialog::resultReady( const QString &collectionId, const Meta::ArtistList &art
         if( !artist->name().isEmpty() )
             m_artists << artist->name();
     }
-
-    m_artists.sort();
 }
 
 void
@@ -215,8 +213,6 @@ TagDialog::resultReady( const QString &collectionId, const Meta::ComposerList &c
         if( !composer->name().isEmpty() )
             m_composers << composer->name();
     }
-
-    m_composers.sort();
 }
 
 void
@@ -229,8 +225,19 @@ TagDialog::resultReady( const QString &collectionId, const Meta::GenreList &genr
         if( !genre->name().isEmpty() )  // Where the heck do the empty genres come from?
             m_genres << genre->name();
     }
+}
 
-    m_genres.sort();
+
+void
+TagDialog::resultReady( const QString &collectionId, const Meta::LabelList &labels )
+{
+    Q_UNUSED( collectionId )
+
+    foreach( const Meta::LabelPtr &label, labels )
+    {
+        if( !label->name().isEmpty() )
+            m_allLabels << label->name();
+    }
 }
 
 void
@@ -238,9 +245,6 @@ TagDialog::dataQueryDone()
 {
     DEBUG_BLOCK
 
-    m_dataQueryMaker->deleteLater();
-    m_dataQueryMaker = 0;
-
     // basically we want to ignore the fact that the fields are being
     // edited because we do it not the user, so it results in empty
     // tags being saved to files---data loss is BAD!
@@ -252,30 +256,52 @@ TagDialog::dataQueryDone()
     // we do this because if we insert items and the contents of the textbox
     // are not in the list, it clears the textbox. which is bad --lfranchi 2.22.09
     QString saveText( ui->kComboBox_artist->lineEdit()->text() );
+    QStringList artists = m_artists.toList();
+    artists.sort();
     ui->kComboBox_artist->clear();
-    ui->kComboBox_artist->insertItems( 0, m_artists );
-    ui->kComboBox_artist->completionObject()->setItems( m_artists );
+    ui->kComboBox_artist->insertItems( 0, artists );
+    ui->kComboBox_artist->completionObject()->setItems( artists );
     ui->kComboBox_artist->lineEdit()->setText( saveText );
 
     saveText = ui->kComboBox_album->lineEdit()->text();
+    QStringList albums = m_albums.toList();
+    albums.sort();
     ui->kComboBox_album->clear();
-    ui->kComboBox_album->insertItems( 0, m_albums );
-    ui->kComboBox_album->completionObject()->setItems( m_albums );
+    ui->kComboBox_album->insertItems( 0, albums );
+    ui->kComboBox_album->completionObject()->setItems( albums );
     ui->kComboBox_album->lineEdit()->setText( saveText );
 
+    saveText = ui->kComboBox_albumArtist->lineEdit()->text();
+    QStringList albumArtists = m_albumArtists.toList();
+    albumArtists.sort();
+    ui->kComboBox_albumArtist->clear();
+    ui->kComboBox_albumArtist->insertItems( 0, albumArtists );
+    ui->kComboBox_albumArtist->completionObject()->setItems( albumArtists );
+    ui->kComboBox_albumArtist->lineEdit()->setText( saveText );
+
     saveText = ui->kComboBox_composer->lineEdit()->text();
+    QStringList composers = m_composers.toList();
+    composers.sort();
     ui->kComboBox_composer->clear();
-    ui->kComboBox_composer->insertItems( 0, m_composers );
-    ui->kComboBox_composer->completionObject()->setItems( m_composers );
+    ui->kComboBox_composer->insertItems( 0, composers );
+    ui->kComboBox_composer->completionObject()->setItems( composers );
     ui->kComboBox_composer->lineEdit()->setText( saveText );
 
     saveText = ui->kComboBox_genre->lineEdit()->text();
+    QStringList genres = m_genres.toList();
+    genres.sort();
     ui->kComboBox_genre->clear();
-    ui->kComboBox_genre->insertItems( 0, m_genres );
-    ui->kComboBox_genre->completionObject()->setItems( m_genres );
+    ui->kComboBox_genre->insertItems( 0, genres );
+    ui->kComboBox_genre->completionObject()->setItems( genres );
     ui->kComboBox_genre->lineEdit()->setText( saveText );
 
-
+    saveText = ui->kComboBox_label->lineEdit()->text();
+    QStringList labels = m_allLabels.toList();
+    labels.sort();
+    ui->kComboBox_label->clear();
+    ui->kComboBox_label->insertItems( 0, labels );
+    ui->kComboBox_label->completionObject()->setItems( labels );
+    ui->kComboBox_label->lineEdit()->setText( saveText );
 
     if( !m_queryMaker )  //track query complete or not necessary
     {
@@ -479,6 +505,12 @@ TagDialog::albumModified()
     checkModified();
 }
 
+inline void TagDialog::albumArtistModified()
+{
+    m_fieldEdited[ "albumartist" ] = true;
+    checkModified();
+}
+
 inline void
 TagDialog::genreModified()
 {
@@ -635,6 +667,12 @@ TagDialog::guessFromFilename() //SLOT
                 ui->kComboBox_album->setItemText( cur, Tags["album"] );
             }
 
+            if( Tags.contains( "albumartist" ) )
+            {
+                cur = ui->kComboBox_albumArtist->currentIndex();
+                ui->kComboBox_albumArtist->setItemText( cur, Tags["albumartist"] );
+            }
+
             if( Tags.contains("track") )
                 ui->qSpinBox_track->setValue( Tags["track"].toInt() );
             
@@ -689,12 +727,10 @@ void TagDialog::init()
     m_removedLabels.clear();
     m_labels.clear();
 
-    m_labelModel = new LabelListModel( m_labels );
+    m_labelModel = new LabelListModel( m_labels, this );
 
     ui->labelsList->setModel( m_labelModel );
-    ui->labelsTab->setEnabled( m_currentTrack->is<Meta::ReadLabelCapability>() );
-
-    loadGlobalLabels();
+    ui->labelsTab->setEnabled( true );
 
     ui->kTabWidget->setCurrentIndex( config.readEntry( "CurrentTab", 0 ) );
 
@@ -704,12 +740,18 @@ void TagDialog::init()
     ui->kComboBox_album->completionObject()->setIgnoreCase( true );
     ui->kComboBox_album->setCompletionMode( KGlobalSettings::CompletionPopup );
 
+    ui->kComboBox_albumArtist->completionObject()->setIgnoreCase( true );
+    ui->kComboBox_albumArtist->setCompletionMode( KGlobalSettings::CompletionPopup );
+
     ui->kComboBox_composer->completionObject()->setIgnoreCase( true );
     ui->kComboBox_composer->setCompletionMode( KGlobalSettings::CompletionPopup );
 
     ui->kComboBox_genre->completionObject()->setIgnoreCase( true );
     ui->kComboBox_genre->setCompletionMode( KGlobalSettings::CompletionPopup );
 
+    ui->kComboBox_label->completionObject()->setIgnoreCase( true );
+    ui->kComboBox_label->setCompletionMode( KGlobalSettings::CompletionPopup );
+
     ui->addButton->setEnabled( false );
     ui->removeButton->setEnabled( false );
 
@@ -745,23 +787,25 @@ void TagDialog::init()
 
     // Connects for modification check
     // only set to overwrite-on-save if the text has changed
-    connect( ui->kLineEdit_title,     SIGNAL( textChanged( const QString& ) ),     SLOT(checkModified()) );
-    connect( ui->kComboBox_composer,  SIGNAL( activated( int ) ),                  SLOT(checkModified()) );
-    connect( ui->kComboBox_composer,  SIGNAL( editTextChanged( const QString& ) ), SLOT(composerModified()) );
-    connect( ui->kComboBox_artist,    SIGNAL( activated( int ) ),                  SLOT(checkModified()) );
-    connect( ui->kComboBox_artist,    SIGNAL( editTextChanged( const QString& ) ), SLOT(artistModified()) );
-    connect( ui->kComboBox_album,     SIGNAL( activated( int ) ),                  SLOT(checkModified()) );
-    connect( ui->kComboBox_album,     SIGNAL( editTextChanged( const QString& ) ), SLOT(albumModified()) );
-    connect( ui->kComboBox_genre,     SIGNAL( activated( int ) ),                  SLOT(checkModified()) );
-    connect( ui->kComboBox_genre,     SIGNAL( editTextChanged( const QString& ) ), SLOT(genreModified()) );
-    connect( ui->kLineEdit_Bpm,       SIGNAL( textChanged( const QString& ) )    , SLOT(bpmModified()) );
-    connect( ui->ratingWidget,        SIGNAL( ratingChanged( int ) ),              SLOT(ratingModified()) );
-    connect( ui->qSpinBox_track,      SIGNAL( valueChanged( int ) ),               SLOT(checkModified()) );
-    connect( ui->qSpinBox_year,       SIGNAL( valueChanged( int ) ),               SLOT(yearModified()) );
-    connect( ui->qSpinBox_score,      SIGNAL( valueChanged( int ) ),               SLOT(scoreModified()) );
-    connect( ui->qPlainTextEdit_comment,   SIGNAL( textChanged() ),                SLOT(commentModified()) );
-    connect( ui->kRichTextEdit_lyrics,    SIGNAL( textChanged() ),                 SLOT(checkModified()) );
-    connect( ui->qSpinBox_discNumber, SIGNAL( valueChanged( int ) ),               SLOT(discNumberModified()) );
+    connect( ui->kLineEdit_title,       SIGNAL( textChanged( const QString& ) ),     SLOT( checkModified() ) );
+    connect( ui->kComboBox_composer,    SIGNAL( activated( int ) ),                  SLOT( checkModified() ) );
+    connect( ui->kComboBox_composer,    SIGNAL( editTextChanged( const QString& ) ), SLOT( composerModified() ) );
+    connect( ui->kComboBox_artist,      SIGNAL( activated( int ) ),                  SLOT( checkModified() ) );
+    connect( ui->kComboBox_artist,      SIGNAL( editTextChanged( const QString& ) ), SLOT( artistModified() ) );
+    connect( ui->kComboBox_album,       SIGNAL( activated( int ) ),                  SLOT( checkModified() ) );
+    connect( ui->kComboBox_album,       SIGNAL( editTextChanged( const QString& ) ), SLOT( albumModified() ) );
+    connect( ui->kComboBox_albumArtist, SIGNAL( activated( int ) ),                  SLOT( checkModified() ) );
+    connect( ui->kComboBox_albumArtist, SIGNAL( editTextChanged( const QString& ) ), SLOT( albumArtistModified() ) );
+    connect( ui->kComboBox_genre,       SIGNAL( activated( int ) ),                  SLOT( checkModified() ) );
+    connect( ui->kComboBox_genre,       SIGNAL( editTextChanged( const QString& ) ), SLOT( genreModified() ) );
+    connect( ui->kLineEdit_Bpm,         SIGNAL( textChanged( const QString& ) )    , SLOT( bpmModified() ) );
+    connect( ui->ratingWidget,          SIGNAL( ratingChanged( int ) ),              SLOT( ratingModified() ) );
+    connect( ui->qSpinBox_track,        SIGNAL( valueChanged( int ) ),               SLOT( checkModified() ) );
+    connect( ui->qSpinBox_year,         SIGNAL( valueChanged( int ) ),               SLOT( yearModified() ) );
+    connect( ui->qSpinBox_score,        SIGNAL( valueChanged( int ) ),               SLOT( scoreModified() ) );
+    connect( ui->qPlainTextEdit_comment,SIGNAL( textChanged() ),                     SLOT( commentModified() ) );
+    connect( ui->kRichTextEdit_lyrics,  SIGNAL( textChanged() ),                     SLOT( checkModified() ) );
+    connect( ui->qSpinBox_discNumber,   SIGNAL( valueChanged( int ) ),               SLOT( discNumberModified() ) );
 
     connect( ui->pushButton_cancel,   SIGNAL( clicked() ), SLOT( cancelPressed() ) );
     connect( ui->pushButton_ok,       SIGNAL( clicked() ), SLOT( accept() ) );
@@ -782,32 +826,36 @@ void TagDialog::init()
 
     ui->pixmap_cover->setContextMenuPolicy( Qt::CustomContextMenu );
     connect( ui->pixmap_cover, SIGNAL( customContextMenuRequested(const QPoint &) ), SLOT( showCoverMenu(const QPoint &) ) );
+
+    connect( ui->pushButton_musicbrainz, SIGNAL( clicked() ), SLOT( musicbrainzTagger() ) );
 }
 
 void
 TagDialog::startDataQuery()
 {
-    Amarok::Collection *coll = CollectionManager::instance()->primaryCollection();
-    if( !coll )
-        return;
+    Collections::QueryMaker *artist = CollectionManager::instance()->queryMaker()->setQueryType( Collections::QueryMaker::Artist );
+    Collections::QueryMaker *album = CollectionManager::instance()->queryMaker()->setQueryType( Collections::QueryMaker::Album );
+    Collections::QueryMaker *composer = CollectionManager::instance()->queryMaker()->setQueryType( Collections::QueryMaker::Composer );
+    Collections::QueryMaker *genre = CollectionManager::instance()->queryMaker()->setQueryType( Collections::QueryMaker::Genre );
+    Collections::QueryMaker *label = CollectionManager::instance()->queryMaker()->setQueryType( Collections::QueryMaker::Label );
 
-    QueryMaker *artist = coll->queryMaker()->setQueryType( QueryMaker::Artist );
-    QueryMaker *album = coll->queryMaker()->setQueryType( QueryMaker::Album );
-    QueryMaker *composer = coll->queryMaker()->setQueryType( QueryMaker::Composer );
-    QueryMaker *genre = coll->queryMaker()->setQueryType( QueryMaker::Genre );
-    QList<QueryMaker*> queries;
-    queries << artist << album << composer << genre;
+    QList<Collections::QueryMaker*> queries;
+    queries << artist << album << composer << genre << label;
 
     //MetaQueryMaker will run multiple different queries just fine as long as we do not use it
     //to set the query type. Configuring the queries is ok though
 
-    m_dataQueryMaker = new MetaQueryMaker( queries );
-    connect( m_dataQueryMaker, SIGNAL( queryDone() ), SLOT( dataQueryDone() ) );
-    connect( m_dataQueryMaker, SIGNAL( newResultReady( QString, Meta::ArtistList ) ), SLOT( resultReady( QString, Meta::ArtistList ) ), Qt::QueuedConnection );
-    connect( m_dataQueryMaker, SIGNAL( newResultReady( QString, Meta::AlbumList ) ), SLOT( resultReady( QString, Meta::AlbumList ) ), Qt::QueuedConnection );
-    connect( m_dataQueryMaker, SIGNAL( newResultReady( QString, Meta::ComposerList ) ), SLOT( resultReady( QString, Meta::ComposerList ) ), Qt::QueuedConnection );
-    connect( m_dataQueryMaker, SIGNAL( newResultReady( QString, Meta::GenreList ) ), SLOT( resultReady( QString, Meta::GenreList ) ), Qt::QueuedConnection );
-    m_dataQueryMaker->run();
+    Collections::QueryMaker *mqm = new Collections::MetaQueryMaker( queries );
+    connect( mqm, SIGNAL( queryDone() ), SLOT( dataQueryDone() ) );
+    connect( mqm, SIGNAL( newResultReady( QString, Meta::ArtistList ) ), SLOT( resultReady( QString, Meta::ArtistList ) ), Qt::QueuedConnection );
+    connect( mqm, SIGNAL( newResultReady( QString, Meta::AlbumList ) ), SLOT( resultReady( QString, Meta::AlbumList ) ), Qt::QueuedConnection );
+    connect( mqm, SIGNAL( newResultReady( QString, Meta::ComposerList ) ), SLOT( resultReady( QString, Meta::ComposerList ) ), Qt::QueuedConnection );
+    connect( mqm, SIGNAL( newResultReady( QString, Meta::GenreList ) ), SLOT( resultReady( QString, Meta::GenreList ) ), Qt::QueuedConnection );
+    connect( mqm, SIGNAL( newResultReady( QString, Meta::LabelList ) ), SLOT( resultReady( QString, Meta::LabelList ) ), Qt::DirectConnection );
+
+    mqm->setAutoDelete( true );
+
+    mqm->run();
 }
 
 
@@ -828,6 +876,9 @@ TagDialog::metadataChanged( Meta::AlbumPtr album )
     // If the metadata of the current album has changed, reload the cover
     if( album == m_currentTrack->album() )
         loadCover();
+
+    // TODO: if the lyrics changed: should we show a warning and ask the user
+    // if he wants to use the new lyrics?
 }
 
 void
@@ -858,8 +909,8 @@ TagDialog::showCoverMenu( const QPoint &pos )
 const QStringList TagDialog::statisticsData()
 {
     QStringList data;
-    QueryMaker *qm = 0;
-    Amarok::Collection *coll = m_currentTrack->collection();
+    Collections::QueryMaker *qm = 0;
+    Collections::Collection *coll = m_currentTrack->collection();
     if( coll )
         qm = coll->queryMaker();
 
@@ -888,7 +939,7 @@ const QStringList TagDialog::statisticsData()
         if( qm )
         {
             // favorite track by this artist
-            qm->setQueryType( QueryMaker::Custom );
+            qm->setQueryType( Collections::QueryMaker::Custom );
             qm->addReturnValue( Meta::valTitle );
             qm->addMatch( trackArtist );
             qm->orderBy( Meta::valPlaycount );
@@ -907,7 +958,7 @@ const QStringList TagDialog::statisticsData()
         {
             // Favorite track on this album
             qm->reset();
-            qm->setQueryType( QueryMaker::Custom );
+            qm->setQueryType( Collections::QueryMaker::Custom );
             qm->addReturnValue( Meta::valTitle );
             qm->addMatch( trackAlbum );
             qm->orderBy( Meta::valPlaycount );
@@ -988,6 +1039,10 @@ void TagDialog::readTags()
         selectOrInsertText( m_currentData.value( Meta::Field::ALBUM ).toString(), ui->kComboBox_album );
     else
         selectOrInsertText( QString(), ui->kComboBox_album );
+    if( m_currentData.contains( Meta::Field::ALBUMARTIST )  )
+        selectOrInsertText( m_currentData.value( Meta::Field::ALBUMARTIST ).toString(), ui->kComboBox_albumArtist );
+    else
+        selectOrInsertText( QString(), ui->kComboBox_albumArtist );
     if( m_currentData.contains( Meta::Field::GENRE ) )
         selectOrInsertText( m_currentData.value( Meta::Field::GENRE ).toString(), ui->kComboBox_genre );
     else
@@ -1029,8 +1084,8 @@ void TagDialog::readTags()
     summaryText += body2cols.arg( i18n("Rating:"), QString::number( static_cast<double>(m_currentTrack->rating())/2.0) );
 
     summaryText += body2cols.arg( i18n("Play Count:"), QString::number( m_currentTrack->playCount() ) );
-    QDate firstPlayed = QDateTime::fromTime_t( m_currentTrack->firstPlayed() ).date();
-    QDate lastPlayed = QDateTime::fromTime_t( m_currentTrack->lastPlayed() ).date();
+    QDate firstPlayed = m_currentTrack->firstPlayed().date();
+    QDate lastPlayed = m_currentTrack->lastPlayed().date();
     summaryText += body2cols.arg( i18n("First Played:"),
                    m_currentTrack->playCount() ? KGlobal::locale()->formatDate( firstPlayed, KLocale::ShortDate ) : i18nc( "When this track was first played", "Never") );
     summaryText += body2cols.arg( i18nc("a single item (singular)", "Last Played:"),
@@ -1068,11 +1123,12 @@ void TagDialog::readTags()
     ui->X->setEnabled( editable ); \
     qobject_cast<KLineEdit*>(ui->X->lineEdit())->setClearButtonShown( editable )
 
-    const bool editable = m_currentTrack->hasCapabilityInterface( Meta::Capability::Editable );
+    const bool editable = m_currentTrack->hasCapabilityInterface( Capabilities::Capability::Editable );
     ui->kLineEdit_title->setEnabled( editable );
     ui->kLineEdit_title->setClearButtonShown( editable );
 
     enableOrDisable( kComboBox_artist );
+    enableOrDisable( kComboBox_albumArtist );
     enableOrDisable( kComboBox_composer );
     enableOrDisable( kComboBox_album );
     enableOrDisable( kComboBox_genre );
@@ -1088,11 +1144,18 @@ void TagDialog::readTags()
     ui->ratingWidget->setEnabled( true );
     ui->qSpinBox_score->setEnabled( true );
     ui->pushButton_guessTags->setEnabled( editable );
+    ui->pushButton_musicbrainz->setEnabled( editable );
 
     if( local )
+    {
         ui->pushButton_guessTags->show();
+        ui->pushButton_musicbrainz->show();
+    }
     else
+    {
        ui->pushButton_guessTags->hide();
+       ui->pushButton_musicbrainz->hide();
+    }
 
     // If it's a local file, write the directory to m_path, else disable the "open in konqui" button
     if ( local )
@@ -1120,6 +1183,7 @@ TagDialog::setMultipleTracksMode()
 
     ui->kComboBox_artist->setItemText( ui->kComboBox_artist->currentIndex(), "" );
     ui->kComboBox_album->setItemText( ui->kComboBox_album->currentIndex(), "" );
+    ui->kComboBox_albumArtist->setItemText( ui->kComboBox_albumArtist->currentIndex(), "" );
     ui->kComboBox_genre->setItemText( ui->kComboBox_genre->currentIndex(), "" );
     ui->kComboBox_composer->setItemText( ui->kComboBox_composer->currentIndex(), "" );
     ui->kLineEdit_title->setText( "" );
@@ -1173,9 +1237,10 @@ TagDialog::readMultipleTracks()
     m_currentData = QVariantMap();
 
     QVariantMap first = dataForTrack( it.peekNext() );
+    QString firstDirectory = it.peekNext()->playableUrl().directory();
 
-    bool artist=true, album=true, genre=true, comment=true, year=true,
-         score=true, rating=true, composer=true, discNumber=true;
+    bool artist=true, album=true, albumArtist = true, genre=true, comment=true, year=true,
+         score=true, rating=true, composer=true, discNumber=true, directory = true;
     int songCount=0, ratingCount=0, ratingSum=0, scoreCount=0;
     double scoreSum = 0.f;
     while( it.hasNext() )
@@ -1190,6 +1255,7 @@ TagDialog::readMultipleTracks()
         }
 
         QVariantMap data = dataForTrack( next );
+        QString currDirectory = next->playableUrl().directory();
         songCount++;
         if ( data.value( Meta::Field::RATING ).toInt() )
         {
@@ -1201,11 +1267,10 @@ TagDialog::readMultipleTracks()
             scoreCount++;
             scoreSum += data.value( Meta::Field::SCORE ).toDouble();
         }
-
         if( !it.peekPrevious()->playableUrl().isLocalFile() )
         {
             // If we have a non local file, don't even lose more time comparing
-            artist = album = genre = comment = year = false;
+            artist = album = genre = comment = year = directory = false;
             score  = rating = composer = discNumber = false;
             continue;
         }
@@ -1213,6 +1278,8 @@ TagDialog::readMultipleTracks()
             artist = false;
         if ( album && data.value( Meta::Field::ALBUM ).toString() != first.value( Meta::Field::ALBUM ).toString() )
             album = false;
+        if ( albumArtist && data.value( Meta::Field::ALBUMARTIST ).toString() != first.value( Meta::Field::ALBUMARTIST ).toString() )
+            albumArtist = false;
         if ( genre && data.value( Meta::Field::GENRE ).toString() != first.value( Meta::Field::GENRE ).toString() )
             genre = false;
         if ( comment && data.value( Meta::Field::COMMENT ).toString() != first.value( Meta::Field::COMMENT ).toString() )
@@ -1223,6 +1290,8 @@ TagDialog::readMultipleTracks()
             composer = false;
         if ( discNumber && data.value( Meta::Field::DISCNUMBER ).toInt() != first.value( Meta::Field::DISCNUMBER ).toInt() )
             discNumber = false;
+        if ( directory && firstDirectory != currDirectory)
+            directory = false;
         //score is double internally, but we only show ints in the tab
         if ( score && data.value( Meta::Field::SCORE ).toInt() != first.value( Meta::Field::SCORE ).toInt() )
             score = false;
@@ -1234,26 +1303,24 @@ TagDialog::readMultipleTracks()
     m_labelModel->setLabels( m_labels );
 
     // Set them in the dialog and in the track ( so we don't break hasChanged() )
-    int cur_item;
     if( artist )
     {
-        cur_item = ui->kComboBox_artist->currentIndex();
         m_currentData.insert( Meta::Field::ARTIST, first.value( Meta::Field::ARTIST ) );
-        ui->kComboBox_artist->completionObject()->insertItems( m_artists );
         selectOrInsertText( first.value( Meta::Field::ARTIST ).toString(), ui->kComboBox_artist );
     }
     if( album )
     {
-        cur_item = ui->kComboBox_album->currentIndex();
         m_currentData.insert( Meta::Field::ALBUM, first.value( Meta::Field::ALBUM ) );
-        ui->kComboBox_album->completionObject()->insertItems( m_albums );
         selectOrInsertText( first.value( Meta::Field::ALBUM ).toString(), ui->kComboBox_album );
     }
+    if( albumArtist )
+    {
+        m_currentData.insert( Meta::Field::ALBUMARTIST, first.value( Meta::Field::ALBUMARTIST ) );
+        selectOrInsertText( first.value( Meta::Field::ALBUMARTIST ).toString(), ui->kComboBox_albumArtist );
+    }
     if( genre )
     {
-        cur_item = ui->kComboBox_genre->currentIndex();
         m_currentData.insert( Meta::Field::GENRE, first.value( Meta::Field::GENRE ) );
-        ui->kComboBox_genre->completionObject()->insertItems( m_genres );
         selectOrInsertText( first.value( Meta::Field::GENRE ).toString(), ui->kComboBox_genre );
     }
     if( comment )
@@ -1263,9 +1330,7 @@ TagDialog::readMultipleTracks()
     }
     if( composer )
     {
-        cur_item = ui->kComboBox_composer->currentIndex();
         m_currentData.insert( Meta::Field::COMPOSER, first.value( Meta::Field::COMPOSER ) );
-        ui->kComboBox_composer->completionObject()->insertItems( m_composers );
         selectOrInsertText( first.value( Meta::Field::COMPOSER ).toString(), ui->kComboBox_composer );
     }
     if( year )
@@ -1278,6 +1343,15 @@ TagDialog::readMultipleTracks()
         m_currentData.insert( Meta::Field::DISCNUMBER, first.value( Meta::Field::DISCNUMBER ) );
         ui->qSpinBox_discNumber->setValue( first.value( Meta::Field::DISCNUMBER ).toInt() );
     }
+    if (directory)
+    {
+        //all files are local & have the same Directory
+        m_path = firstDirectory;
+        ui->locationLabel->show();
+        ui->kLineEdit_location->show();
+        ui->pushButton_open->show();
+        ui->kLineEdit_location->setText( firstDirectory );
+    }
     if( score )
     {
         m_currentData.insert( Meta::Field::SCORE, first.value( Meta::Field::SCORE ) );
@@ -1334,6 +1408,7 @@ TagDialog::changes()
     bool modified = false;
     modified |= !equalString( ui->kComboBox_artist->lineEdit()->text(), m_currentData.value( Meta::Field::ARTIST ).toString() );
     modified |= !equalString( ui->kComboBox_album->lineEdit()->text(), m_currentData.value( Meta::Field::ALBUM ).toString() );
+    modified |= !equalString( ui->kComboBox_albumArtist->lineEdit()->text(), m_currentData.value( Meta::Field::ALBUMARTIST ).toString() );
     modified |= !equalString( ui->kComboBox_genre->lineEdit()->text(), m_currentData.value( Meta::Field::GENRE ).toString() );
     modified |= ui->qSpinBox_year->value()  != m_currentData.value( Meta::Field::YEAR ).toInt();
     modified |= !equalString( ui->kLineEdit_Bpm->text() , m_currentData.value( Meta::Field::BPM ).toString() );
@@ -1395,6 +1470,8 @@ TagDialog::storeTags( const Meta::TrackPtr &track )
             map.insert( Meta::Field::ARTIST, ui->kComboBox_artist->currentText() );
         if ( !track->album() || ui->kComboBox_album->currentText() != track->album()->name() )
             map.insert( Meta::Field::ALBUM, ui->kComboBox_album->currentText() );
+        if ( !track->album()->hasAlbumArtist() || ui->kComboBox_albumArtist->currentText() != track->album()->albumArtist()->name() )
+            map.insert( Meta::Field::ALBUMARTIST, ui->kComboBox_albumArtist->currentText() );
         if ( ui->qPlainTextEdit_comment->toPlainText() != track->comment() )
             map.insert( Meta::Field::COMMENT, ui->qPlainTextEdit_comment->toPlainText() );
         if ( !track->genre() || ui->kComboBox_genre->currentText() != track->genre()->name() )
@@ -1468,19 +1545,31 @@ TagDialog::storeLabels( Meta::TrackPtr track, const QStringList &removedLabels,
 {
     DEBUG_BLOCK
 
-    Meta::WriteLabelCapability *wlc = track->create<Meta::WriteLabelCapability>();
-
-    if( !wlc )
+    if( track )
     {
-        debug() << "Unable to get a write label capability, aborting";
-        return;
-    }
-    wlc->setLabels( removedLabels, newLabels );
-    delete wlc;
+        QHash<QString, Meta::LabelPtr> labelMap;
+        foreach( const Meta::LabelPtr &label, track->labels() )
+        {
+            labelMap.insert( label->name(), label );
+        }
 
-    Meta::ReadLabelCapability *rlc = track->create<Meta::ReadLabelCapability>();
-    if( rlc )
-        rlc->fetchLabels(); // If new labels are set, we need to fetchLabels() again to update the cache.  This should probably be done centrally..
+        foreach( const QString &oldLabel, removedLabels )
+        {
+            if( labelMap.contains( oldLabel ) )
+            {
+                Meta::LabelPtr label = labelMap.value( oldLabel );
+                track->removeLabel( label );
+            }
+        }
+
+        foreach( const QString &newLabel, newLabels )
+        {
+            if( !labelMap.contains( newLabel ) )
+            {
+                track->addLabel( newLabel );
+            }
+        }
+    }
 }
 
 
@@ -1508,7 +1597,7 @@ TagDialog::labelsForTrack( Meta::TrackPtr track )
 {
     DEBUG_BLOCK
 
-    Meta::ReadLabelCapability *ric = track->create<Meta::ReadLabelCapability>();
+    QScopedPointer<Capabilities::ReadLabelCapability> ric( track->create<Capabilities::ReadLabelCapability>() );
     if( !ric )
     {
         debug() << "No Read Label Capability found, no labels available.";
@@ -1523,31 +1612,17 @@ TagDialog::loadLabels( Meta::TrackPtr track )
 {
     if( track )
     {
-        Meta::ReadLabelCapability *ric = track->create<Meta::ReadLabelCapability>();
-        if( !ric )
+        QStringList labelNames;
+        foreach( const Meta::LabelPtr &label, track->labels() )
         {
-            debug() << "No Read Label Capability found, no labels available.";
-            return;
+            labelNames << label->name();
         }
-        connect( ric, SIGNAL(labelsFetched(QStringList)), SLOT(trackLabelsFetched(QStringList)));
-        ric->fetchLabels();
+        m_labels = labelNames;
+        m_labelModel->setLabels( labelNames );
+        ui->labelsList->update();
     }
 }
 
-void
-TagDialog::loadGlobalLabels()
-{
-    Meta::ReadLabelCapability *ric = m_currentTrack->create<Meta::ReadLabelCapability>();
-    if( !ric )
-    {
-        debug() << "No Read Label Capability found, no labels available.";
-        return;
-    }
-    connect( ric, SIGNAL(labelsFetched(QStringList)), SLOT(globalLabelsFetched(QStringList)));
-    ric->fetchGlobalLabels();
-}
-
-
 void
 TagDialog::trackLabelsFetched( QStringList labels )
 {
@@ -1557,17 +1632,6 @@ TagDialog::trackLabelsFetched( QStringList labels )
     ui->labelsList->update();
 }
 
-void
-TagDialog::globalLabelsFetched( QStringList labels )
-{
-    sender()->deleteLater();
-    ui->kComboBox_label->addItems( labels );
-    ui->kComboBox_label->completionObject()->insertItems( labels );
-    ui->kComboBox_label->update();
-    ui->kComboBox_label->setCurrentIndex( -1 );
-}
-
-
 QVariantMap
 TagDialog::dataForTrack( const Meta::TrackPtr &track )
 {
@@ -1637,16 +1701,16 @@ TagDialog::saveTags()
             emit lyricsChanged( track->uidUrl() );
         }
 
-        Meta::EditCapability *ec = track->create<Meta::EditCapability>();
+        QScopedPointer<Capabilities::EditCapability> ec( track->create<Capabilities::EditCapability>() );
         if( !ec )
         {
-            debug() << "Track does not have Meta::EditCapability. Aborting loop.";
+            debug() << "Track does not have Capabilities::EditCapability. Aborting loop.";
             continue;
         }
         if( !ec->isEditable() )
         {
             debug() << "Track not editable. Aborting loop.";
-            The::statusBar()->shortMessage( i18n( "Writing to file failed. Please check permissions and available disc space." ) );
+            Amarok::Components::logger()->shortMessage( i18n( "Writing to file failed. Please check permissions and available disc space." ) );
             continue;
         }
 
@@ -1660,7 +1724,8 @@ TagDialog::saveTags()
              data.contains( Meta::Field::GENRE ) || data.contains( Meta::Field::COMPOSER ) ||
              data.contains( Meta::Field::YEAR ) || data.contains( Meta::Field::TRACKNUMBER ) ||
              data.contains( Meta::Field::TRACKNUMBER ) || data.contains( Meta::Field::DISCNUMBER ) ||
-             data.contains( Meta::Field::BPM )
+             data.contains( Meta::Field::BPM ) || data.contains( Meta::Field::UNIQUEID ) ||
+             data.contains( Meta::Field::ALBUMARTIST )
              )
         {
 
@@ -1680,13 +1745,17 @@ TagDialog::saveTags()
             if( data.contains( Meta::Field::COMPOSER ) )
                 ec->setComposer( data.value( Meta::Field::COMPOSER ).toString() );
             if( data.contains( Meta::Field::YEAR ) )
-                ec->setYear( data.value( Meta::Field::YEAR ).toString() );
+                ec->setYear( data.value( Meta::Field::YEAR ).toInt() );
             if( data.contains( Meta::Field::TRACKNUMBER ) )
                 ec->setTrackNumber( data.value( Meta::Field::TRACKNUMBER ).toInt() );
             if( data.contains( Meta::Field::DISCNUMBER ) )
                 ec->setDiscNumber( data.value( Meta::Field::DISCNUMBER ).toInt() );
             if( data.contains( Meta::Field::BPM ) )
                 ec->setBpm( data.value( Meta::Field::BPM ).toDouble() );
+            if( data.contains( Meta::Field::UNIQUEID ) )
+                ec->setUidUrl( data.value( Meta::Field::UNIQUEID ).toString() );
+            if( data.contains( Meta::Field::ALBUMARTIST ) )
+                ec->setAlbumArtist( data.value( Meta::Field::ALBUMARTIST ).toString() );
             ec->endMetaDataUpdate();
         }
     }
@@ -1697,7 +1766,7 @@ TagDialog::saveTags()
 
     foreach( Meta::TrackPtr track, m_tracks )
     {
-        Meta::UpdateCapability *uc = track->create<Meta::UpdateCapability>();
+        QScopedPointer<Capabilities::UpdateCapability> uc( track->create<Capabilities::UpdateCapability>() );
         if( !uc )
         {
             continue;
@@ -1715,10 +1784,9 @@ TagDialog::saveTags()
 
     // use the representative tracks to send updated signals
 
-    foreach( Meta::TrackPtr track, collectionsToUpdateMap.values() )
+    foreach( Meta::TrackPtr track, collectionsToUpdateMap )
     {
-        Meta::UpdateCapability *uc = track->create<Meta::UpdateCapability>();
-
+        QScopedPointer<Capabilities::UpdateCapability> uc( track->create<Capabilities::UpdateCapability>() );
         uc->collectionUpdated();
     }
 }
@@ -1749,6 +1817,12 @@ TagDialog::applyToAllTracks()
             changed |= TagDialog::TAGSCHANGED;
         }
 
+        if( m_fieldEdited.contains( "albumartist" ) && m_fieldEdited[ "albumartist" ] )
+        {
+            data.insert( Meta::Field::ALBUMARTIST, ui->kComboBox_albumArtist->currentText() );
+            changed |= TagDialog::TAGSCHANGED;
+        }
+
         if( m_fieldEdited.contains( "genre" ) && m_fieldEdited[ "genre" ] )
         {
             data.insert( Meta::Field::GENRE, ui->kComboBox_genre->currentText() );
@@ -1818,5 +1892,42 @@ TagDialog::selectOrInsertText( const QString &text, QComboBox *comboBox )
     }
 }
 
+void
+TagDialog::musicbrainzTagger()
+{
+    DEBUG_BLOCK
+
+    MusicBrainzTagger *dialog = new MusicBrainzTagger( m_tracks, this );
+    dialog->setWindowTitle( i18n( "MusicBrainz Tagger" ) );
+    connect( dialog, SIGNAL( sendResult( const QMap<Meta::TrackPtr,QVariantMap> ) ),
+             this, SLOT( musicbrainzTaggerResult( const QMap<Meta::TrackPtr,QVariantMap> ) ) );
+    dialog->show();
+}
+
+void
+TagDialog::musicbrainzTaggerResult( const QMap<Meta::TrackPtr, QVariantMap > result )
+{
+    if( result.isEmpty() )
+        return;
+    foreach( Meta::TrackPtr track, result.keys() )
+    {
+        QVariantMap data = result.value( track );
+        storeTags( track, TagDialog::TAGSCHANGED, data );
+    }
+
+    if( m_perTrack )
+    {
+        loadTags( m_currentTrack );
+        readTags();
+    }
+    else
+    {
+        setMultipleTracksMode();
+        readMultipleTracks();
+        enableItems();
+    }
+    checkModified();
+}
+
 #include "TagDialog.moc"
 
