@@ -22,6 +22,7 @@
 #include <QRegExp>
 
 #define album_color Qt::red
+#define albumartist_color Qt::blue
 #define artist_color Qt::blue
 #define comment_color Qt::gray
 #define composer_color Qt::magenta
@@ -85,18 +86,18 @@ class TagGuesser
 
         struct TagStruct
         {
-            QString type;                   //! Type of the Tag (e.g. "artist")
-            QString tag;                    //! Tag itself
+            QString type;                   //!< Type of the Tag (e.g. "artist")
+            QString tag;                    //!< Tag itself
         };
         
-        QMap<QString,QString> m_tags;       //! Taglist (e.g. <"artist","some artist">
-        QMap<int,TagStruct> m_sortedTags;   //! Taglist sorted after their occurence in the filename
-        bool m_guessed;                     //! Is true when guessing was done
-        QString m_fileName;                 //! Filename to be guessed from
-        QString m_schema;                   //! Schema after which should be guessed
-        int m_caseOptions;                  //! Case options to change tags after
-        bool m_cutTrailingSpaces;           //! Whether trailing spaces should be cut from tags
-        bool m_convertUnderscores;          //! Whether underscores should be converted to spaces
+        QMap<QString,QString> m_tags;       //!< Taglist (e.g. <"artist","some artist">
+        QMap<int,TagStruct> m_sortedTags;   //!< Taglist sorted after their occurence in the filename
+        bool m_guessed;                     //!< Is true when guessing was done
+        QString m_fileName;                 //!< Filename to be guessed from
+        QString m_schema;                   //!< Schema after which should be guessed
+        int m_caseOptions;                  //!< Case options to change tags after
+        bool m_cutTrailingSpaces;           //!< Whether trailing spaces should be cut from tags
+        bool m_convertUnderscores;          //!< Whether underscores should be converted to spaces
 
         /**
         *   Converts a tag to specific case type
