@@ -22,11 +22,11 @@
 
 #include "amarokconfig.h"
 #include "App.h"
-#include "Collection.h"
-#include "CollectionManager.h"
-#include "Debug.h"
+#include "core/collections/Collection.h"
+#include "core-impl/collections/support/CollectionManager.h"
+#include "core/support/Debug.h"
 #include "DynamicModel.h"
-#include "MetaQueryMaker.h"
+#include "core/collections/MetaQueryMaker.h"
 #include "playlist/PlaylistModelStack.h"
 #include "statusbar/StatusBar.h"
 
@@ -35,7 +35,13 @@
 
 
 // The bigger this is, the more accurate the result will be. Big is good.
-const int Dynamic::BiasedPlaylist::BUFFER_SIZE = 100;
+// On the other hand optimizing a ridiculous large playlist for nothing
+// is just a waste of processing power.
+// expecially since it seems that the buffers are deleted
+// every time the playlist changes (e.g. after the rating changed)
+// So Small is good.
+// Pick your poison...
+const int Dynamic::BiasedPlaylist::BUFFER_SIZE = 50;
 
 
 Dynamic::BiasedPlaylist*
@@ -70,7 +76,7 @@ Dynamic::BiasedPlaylist::nameFromXml( QDomElement e )
 }
 
 
-Dynamic::BiasedPlaylist::BiasedPlaylist( QString title, QList<Bias*> biases, Amarok::Collection* collection )
+Dynamic::BiasedPlaylist::BiasedPlaylist( QString title, QList<Bias*> biases, Collections::Collection* collection )
     : DynamicPlaylist(collection)
     , m_numRequested( 0 )
     , m_biases( biases )
@@ -86,15 +92,7 @@ Dynamic::BiasedPlaylist::~BiasedPlaylist()
 {
     DEBUG_BLOCK
 
-    if( m_solver )
-    {
-        m_solver->requestAbort();
-
-        while( !m_solver->isFinished() )
-            usleep( 20000 ); // Sleep 20 msec
-
-        delete m_solver;
-    }
+    requestAbort();
 }
 
 QDomElement
@@ -115,14 +113,11 @@ Dynamic::BiasedPlaylist::xml() const
 void
 Dynamic::BiasedPlaylist::requestAbort()
 {
-    if( m_solver )
-        m_solver->requestAbort();
-}
-
-void
-Dynamic::BiasedPlaylist::setContext( Meta::TrackList context )
-{
-    m_context = context;
+    if( m_solver ) {
+        m_solver.data()->requestAbort();
+        disconnect(m_solver.data(), 0, this, 0);
+        m_solver.clear();
+    }
 }
 
 void
@@ -130,24 +125,26 @@ Dynamic::BiasedPlaylist::startSolver( bool withStatusBar )
 {
     DEBUG_BLOCK
     debug() << "BiasedPlaylist in:" << QThread::currentThreadId();
-    
+
     if( !m_solver )
     {
         BiasSolver::setUniverseCollection( m_collection );
         debug() << "assigning new m_solver";
 
-        m_solver = new BiasSolver( BUFFER_SIZE, m_biases, m_context );
-        connect( m_solver, SIGNAL( done( ThreadWeaver::Job* ) ), SLOT( solverFinished(ThreadWeaver::Job* ) ) );
+        m_solver = new BiasSolver( BUFFER_SIZE, m_biases, getContext() );
+        m_solver.data()->setAutoDelete(true);
+        connect( m_solver.data(), SIGNAL(readyToRun()), SLOT(solverReady()) );
+        connect( m_solver.data(), SIGNAL(done(ThreadWeaver::Job*)), SLOT(solverFinished()) );
+        connect( m_solver.data(), SIGNAL(failed(ThreadWeaver::Job*)), SLOT(solverFinished()) );
 
         if( withStatusBar )
         {
-            The::statusBar()->newProgressOperation( m_solver,  i18n( "Generating playlist..." ) );
+            The::statusBar()->newProgressOperation( m_solver.data(), i18n( "Generating playlist..." ) );
 
-            connect( m_solver, SIGNAL(statusUpdate(int)), SLOT(updateStatus(int)) );
+            connect( m_solver.data(), SIGNAL(statusUpdate(int)), SLOT(updateStatus(int)) );
         }
 
-        connect( m_solver, SIGNAL(readyToRun()), SLOT(solverReady()) );
-        m_solver->prepareToRun();
+        m_solver.data()->prepareToRun();
         debug() << "called prepareToRun";
     }
     else
@@ -159,14 +156,14 @@ Dynamic::BiasedPlaylist::solverReady()
 {
     debug() << "ENQUEUEING new m_solver!" << m_solver;
     if( m_solver )
-        ThreadWeaver::Weaver::instance()->enqueue( m_solver );
+        ThreadWeaver::Weaver::instance()->enqueue( m_solver.data() );
 }
 
 
 void
 Dynamic::BiasedPlaylist::updateStatus( int progress )
 {
-    The::statusBar()->setProgress( m_solver, progress );
+    The::statusBar()->setProgress( m_solver.data(), progress );
 }
 
 void
@@ -174,13 +171,10 @@ Dynamic::BiasedPlaylist::requestTracks( int n )
 {
     debug() << "Requesting " << n << " tracks.";
 
-
-    if( n <= 0 )
-        emit tracksReady( Meta::TrackList() );
-
-    m_requestCache.clear();
-    m_numRequested = n;
-
+    {
+        QMutexLocker locker(&m_bufferMutex);
+        m_numRequested = n;
+    }
     handleRequest();
 }
 
@@ -189,14 +183,13 @@ Dynamic::BiasedPlaylist::recalculate()
 {
     DEBUG_BLOCK
     if ( AmarokConfig::dynamicMode() && !m_solver ) {
-        m_buffer.clear();
-        if ( m_backbufferMutex.tryLock() ) {
-            m_backbuffer.clear();
-            m_backbufferMutex.unlock();
+        {
+            QMutexLocker locker(&m_bufferMutex);
+            m_buffer.clear();
         }
 
-        getContext();
-        startSolver( true );
+        if( m_numRequested > 0 )
+            startSolver( true );
     }
 }
 
@@ -229,76 +222,74 @@ Dynamic::BiasedPlaylist::handleRequest()
 {
     DEBUG_BLOCK
 
-    if( m_buffer.isEmpty() )
-    {
-        m_backbufferMutex.lock();
-        m_buffer = m_backbuffer;
-        m_backbuffer.clear();
-        startSolver( true );
-        m_backbufferMutex.unlock();
-    }
+    QMutexLocker locker(&m_bufferMutex);
 
-    while( m_buffer.size() && m_numRequested-- )
-        m_requestCache.append( m_buffer.takeLast() );
-
-    if( m_numRequested <= 0 )
+    // if we have enough tracks, submit them.
+    if( m_buffer.count() >= m_numRequested )
     {
-        m_numRequested = 0;
-        debug() << "Returning " << m_requestCache.size() << " tracks.";
-        emit tracksReady( m_requestCache );
+        Meta::TrackList tracks;
+        while( !m_buffer.isEmpty() && m_numRequested-- )
+            tracks.append( m_buffer.takeFirst() );
+        locker.unlock();
+
+        debug() << "Returning " << tracks.size() << " tracks.";
+        emit tracksReady( tracks );
     }
-    // otherwise, we ran out of buffer
     else
     {
-        m_backbufferMutex.lock();
-        m_buffer = m_backbuffer;
-        m_backbuffer.clear();
+        locker.unlock();
+        // otherwise, we ran out of buffer
         startSolver( true );
-        m_backbufferMutex.unlock();
     }
 }
 
 
 void
-Dynamic::BiasedPlaylist::solverFinished( ThreadWeaver::Job* job )
+Dynamic::BiasedPlaylist::solverFinished()
 {
     DEBUG_BLOCK
 
     if( !m_solver )
         return;
 
-    bool success;
-    The::statusBar()->endProgressOperation( m_solver );
-    m_backbufferMutex.lock();
-    m_backbuffer = m_solver->solution();
-    m_backbufferMutex.unlock();
-    success = m_solver->success();
-    job->deleteLater();
-    m_solver = 0;
+    The::statusBar()->endProgressOperation( m_solver.data() );
 
-    // empty collection, or it was aborted
-    if( !success || m_backbuffer.isEmpty() )
+    bool success = m_solver.data()->success();
+    if( success )
     {
-        m_requestCache.clear();
-        m_numRequested = 0;
-
-        emit tracksReady( Meta::TrackList() );
+        QMutexLocker locker(&m_bufferMutex);
+        m_buffer.append( m_solver.data()->solution() );
     }
-    else if( m_numRequested > 0 )
-        handleRequest();
+
+    m_solver.data()->deleteLater();
+
+    // empty collection just give up.
+    if(m_buffer.isEmpty())
+    {
+        m_numRequested = 0;
+    }
+
+    handleRequest();
 }
 
 
-void
+Meta::TrackList
 Dynamic::BiasedPlaylist::getContext()
 {
-    m_context.clear();
+    Meta::TrackList context;
 
     int i = qMax( 0, The::playlist()->activeRow() );
 
-    for( ; i < The::playlist()->rowCount(); ++i )
+    for( ; i < The::playlist()->qaim()->rowCount(); ++i )
     {
-        m_context.append( The::playlist()->trackAt(i) );
+        context.append( The::playlist()->trackAt(i) );
     }
+
+    {
+        QMutexLocker locker(&m_bufferMutex);
+        context.append( m_buffer );
+    }
+
+    return context;
 }
 
