@@ -1,29 +1,31 @@
-/***************************************************************************
- * Copyright (c) 2008-2009  Nikolaj Hald Nielsen <nhnFreespirit@gmail.com> *
- *           (c) 2009  Seb Ruiz <ruiz@kde.org>                             *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- *   This program is distributed in the hope that it will be useful,       *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
- *   GNU General Public License for more details.                          *
- *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
- ***************************************************************************/
- 
+/****************************************************************************************
+ * Copyright (c) 2008-2009 Nikolaj Hald Nielsen <nhn@kde.org>                           *
+ * Copyright (c) 2009 Seb Ruiz <ruiz@kde.org>                                           *
+ * Copyright (c) 2010 Oleksandr Khayrullin <saniokh@gmail.com>                          *
+ *                                                                                      *
+ * This program is free software; you can redistribute it and/or modify it under        *
+ * the terms of the GNU General Public License as published by the Free Software        *
+ * Foundation; either version 2 of the License, or (at your option) any later           *
+ * version.                                                                             *
+ *                                                                                      *
+ * This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
+ * PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
+ *                                                                                      *
+ * You should have received a copy of the GNU General Public License along with         *
+ * this program.  If not, see <http://www.gnu.org/licenses/>.                           *
+ ****************************************************************************************/
+
 #include "LayoutManager.h"
 
-#include "Amarok.h"
-#include "Debug.h"
+#include "core/support/Amarok.h"
+#include "core/support/Components.h"
+#include "core/support/Debug.h"
+#include "core/interfaces/Logger.h"
 #include "playlist/PlaylistDefines.h"
+#include "playlist/PlaylistModelStack.h"
 
+#include <KConfigGroup>
 #include <KMessageBox>
 #include <KStandardDirs>
 #include <KUrl>
@@ -56,6 +58,9 @@ LayoutManager::LayoutManager()
 
     KConfigGroup config = Amarok::config("Playlist Layout");
     m_activeLayout = config.readEntry( "CurrentLayout", "Default" );
+    if( !layouts().contains( m_activeLayout ) )
+        m_activeLayout = "Default";
+    Playlist::ModelStack::instance()->groupingProxy()->setGroupingCategory( activeLayout().groupBy() );
 }
 
 QStringList LayoutManager::layouts() const
@@ -68,13 +73,54 @@ void LayoutManager::setActiveLayout( const QString &layout )
     m_activeLayout = layout;
     Amarok::config( "Playlist Layout" ).writeEntry( "CurrentLayout", m_activeLayout );
     emit( activeLayoutChanged() );
+
+    //Change the grouping style to that of this layout.
+    Playlist::ModelStack::instance()->groupingProxy()->setGroupingCategory( activeLayout().groupBy() );
+
 }
 
 void LayoutManager::setPreviewLayout( const PlaylistLayout &layout )
 {
-    m_activeLayout = PREVIEW_LAYOUT; 
+    DEBUG_BLOCK
+    m_activeLayout = PREVIEW_LAYOUT;
     m_previewLayout = layout;
     emit( activeLayoutChanged() );
+
+    //Change the grouping style to that of this layout.
+    Playlist::ModelStack::instance()->groupingProxy()->setGroupingCategory( activeLayout().groupBy() );
+}
+
+void LayoutManager::updateCurrentLayout( const PlaylistLayout &layout )
+{
+    //Do not store preview layouts.
+    if ( m_activeLayout == PREVIEW_LAYOUT )
+        return;
+
+    if ( m_layouts.value( m_activeLayout ).isEditable() )
+    {
+        addUserLayout( m_activeLayout, layout );
+        setActiveLayout( m_activeLayout );
+    }
+    else
+    {
+        //create a writable copy of this layout. (Copy on Write)
+        QString newLayoutName = i18n( "copy of %1", m_activeLayout );
+        QString orgCopyName = newLayoutName;
+
+        int copyNumber = 1;
+        QStringList existingLayouts = LayoutManager::instance()->layouts();
+        while( existingLayouts.contains( newLayoutName ) )
+        {
+            copyNumber++;
+            newLayoutName = i18nc( "adds a copy number to a generated name if the name already exists, for instance 'copy of Foo 2' if 'copy of Foo' is taken", "%1 %2", orgCopyName, copyNumber );
+        }
+
+
+        Amarok::Components::logger()->longMessage( i18n( "Current layout '%1' is read only. Creating a new layout '%2' with your changes and setting this as active", m_activeLayout, newLayoutName ) );
+
+        addUserLayout( newLayoutName, layout );
+        setActiveLayout( newLayoutName );
+    }
 }
 
 PlaylistLayout LayoutManager::activeLayout() const
@@ -100,7 +146,6 @@ void LayoutManager::loadUserLayouts()
     for ( int i = 0; i < list.size(); ++i )
     {
         QFileInfo fileInfo = list.at(i);
-        debug() << "found user file: " << fileInfo.fileName();
         loadLayouts( layoutsDir.filePath( fileInfo.fileName() ), true );
     }
 }
@@ -108,13 +153,14 @@ void LayoutManager::loadUserLayouts()
 void LayoutManager::loadDefaultLayouts()
 {
     const KUrl url( KStandardDirs::locate( "data", "amarok/data/" ) );
-    QString configFile = url.path() + "DefaultPlaylistLayouts.xml";
+    QString configFile = url.toLocalFile() + "DefaultPlaylistLayouts.xml";
     loadLayouts( configFile, false );
 }
 
 
 void LayoutManager::loadLayouts( const QString &fileName, bool user )
 {
+    DEBUG_BLOCK
     QDomDocument doc( "layouts" );
 
     if ( !QFile::exists( fileName ) )
@@ -148,12 +194,26 @@ void LayoutManager::loadLayouts( const QString &fileName, bool user )
         index++;
 
         QString layoutName = layout.toElement().attribute( "name", "" );
+        debug() << "loading layout " << layoutName;
+
         PlaylistLayout currentLayout;
         currentLayout.setEditable( user );
+        currentLayout.setInlineControls( layout.toElement().attribute( "inline_controls", "false" ).compare( "true", Qt::CaseInsensitive ) == 0 );
+        currentLayout.setTooltips( layout.toElement().attribute( "tooltips", "false" ).compare( "true", Qt::CaseInsensitive ) == 0 );
 
-        currentLayout.setHead( parseItemConfig( layout.toElement().firstChildElement( "group_head" ) ) );
-        currentLayout.setBody( parseItemConfig( layout.toElement().firstChildElement( "group_body" ) ) );
-        currentLayout.setSingle( parseItemConfig( layout.toElement().firstChildElement( "single_track" ) ) );
+        //For backwards compatibility, if a grouping is not set in the XML file assume "group by album" (which was previously the default)
+        currentLayout.setGroupBy( layout.toElement().attribute( "group_by", "Album" ) );
+        debug() << "grouping mode is: " << layout.toElement().attribute( "group_by", "Album" );
+
+
+        currentLayout.setLayoutForPart( PlaylistLayout::Head, parseItemConfig( layout.toElement().firstChildElement( "group_head" ) ) );
+        currentLayout.setLayoutForPart( PlaylistLayout::StandardBody, parseItemConfig( layout.toElement().firstChildElement( "group_body" ) ) );
+        QDomElement variousArtistsXML = layout.toElement().firstChildElement( "group_variousArtistsBody" );
+        if ( !variousArtistsXML.isNull() )
+            currentLayout.setLayoutForPart( PlaylistLayout::VariousArtistsBody, parseItemConfig( variousArtistsXML ) );
+        else    // Handle old custom layout XMLs
+            currentLayout.setLayoutForPart( PlaylistLayout::VariousArtistsBody, parseItemConfig( layout.toElement().firstChildElement( "group_body" ) ) );
+        currentLayout.setLayoutForPart( PlaylistLayout::Single, parseItemConfig( layout.toElement().firstChildElement( "single_track" ) ) );
 
         if ( !layoutName.isEmpty() )
             m_layouts.insert( layoutName, currentLayout );
@@ -193,9 +253,10 @@ LayoutItemConfig LayoutManager::parseItemConfig( const QDomElement &elem ) const
             qreal size = elementNode.toElement().attribute( "size", "0.0" ).toDouble();
             bool bold = ( elementNode.toElement().attribute( "bold", "false" ).compare( "true", Qt::CaseInsensitive ) == 0 );
             bool italic = ( elementNode.toElement().attribute( "italic", "false" ).compare( "true", Qt::CaseInsensitive ) == 0 );
+            bool underline = ( elementNode.toElement().attribute( "underline", "false" ).compare( "true", Qt::CaseInsensitive ) == 0 );
             QString alignmentString = elementNode.toElement().attribute( "alignment", "left" );
             Qt::Alignment alignment;
-            
+
 
             if ( alignmentString.compare( "left", Qt::CaseInsensitive ) == 0 )
                 alignment = Qt::AlignLeft | Qt::AlignVCenter;
@@ -204,7 +265,8 @@ LayoutItemConfig LayoutManager::parseItemConfig( const QDomElement &elem ) const
             else
                 alignment = Qt::AlignCenter| Qt::AlignVCenter;
 
-            row.addElement( LayoutItemConfigRowElement( value, size, bold, italic, alignment, prefix, sufix ) );
+            row.addElement( LayoutItemConfigRowElement( value, size, bold, italic, underline,
+                                                        alignment, prefix, sufix ) );
         }
 
         config.addRow( row );
@@ -242,14 +304,22 @@ void LayoutManager::addUserLayout( const QString &name, PlaylistLayout layout )
     QDomElement body = doc.createElement( "body" );
     QDomElement single = doc.createElement( "single" );
 
-    newLayout.appendChild( createItemElement( doc, "single_track", layout.single() ) );
-    newLayout.appendChild( createItemElement( doc, "group_head", layout.head() ) );
-    newLayout.appendChild( createItemElement( doc, "group_body", layout.body() ) );
+    newLayout.appendChild( createItemElement( doc, "single_track", layout.layoutForPart( PlaylistLayout::Single ) ) );
+    newLayout.appendChild( createItemElement( doc, "group_head", layout.layoutForPart( PlaylistLayout::Head ) ) );
+    newLayout.appendChild( createItemElement( doc, "group_body", layout.layoutForPart( PlaylistLayout::StandardBody ) ) );
+    newLayout.appendChild( createItemElement( doc, "group_variousArtistsBody", layout.layoutForPart( PlaylistLayout::VariousArtistsBody ) ) );
+
+    if( layout.inlineControls() )
+        newLayout.setAttribute( "inline_controls", "true" );
+
+    if( layout.tooltips() )
+        newLayout.setAttribute( "tooltips", "true" );
+
+    newLayout.setAttribute( "group_by", layout.groupBy() );
 
-    debug() << doc.toString();
     QDir layoutsDir = QDir( Amarok::saveLocation( "playlist_layouts/" ) );
 
-    //make sure that this dir exists
+    //make sure that this directory exists
     if ( !layoutsDir.exists() )
         layoutsDir.mkpath( Amarok::saveLocation( "playlist_layouts/" ) );
 
@@ -280,10 +350,13 @@ QDomElement LayoutManager::createItemElement( QDomDocument doc, const QString &n
             LayoutItemConfigRowElement element = row.element( j );
             QDomElement elementElement = doc.createElement( "element" );
 
+            elementElement.setAttribute ( "prefix", element.prefix() );
+            elementElement.setAttribute ( "suffix", element.suffix() );
             elementElement.setAttribute ( "value", internalColumnNames[element.value()] );
             elementElement.setAttribute ( "size", QString::number( element.size() ) );
             elementElement.setAttribute ( "bold", element.bold() ? "true" : "false" );
             elementElement.setAttribute ( "italic", element.italic() ? "true" : "false" );
+            elementElement.setAttribute ( "underline", element.underline() ? "true" : "false" );
 
             QString alignmentString;
             if ( element.alignment() & Qt::AlignLeft )
@@ -292,7 +365,7 @@ QDomElement LayoutManager::createItemElement( QDomDocument doc, const QString &n
                 alignmentString = "right";
             else
                 alignmentString = "center";
-            
+
             elementElement.setAttribute ( "alignment", alignmentString );
 
             rowElement.appendChild( elementElement );
@@ -322,10 +395,9 @@ void LayoutManager::deleteLayout( const QString &layout )
     {
         QDir layoutsDir = QDir( Amarok::saveLocation( "playlist_layouts/" ) );
         QString xmlFile = layoutsDir.path() + '/' + layout + ".xml";
-        debug() << "deleting file: " << xmlFile;
-       
+
         if ( !QFile::remove( xmlFile ) )
-            debug() << "error deleting file....";
+            debug() << "error deleting file" << xmlFile;
 
         m_layouts.remove( layout );
         m_layoutNames.removeAll( layout );
@@ -373,23 +445,23 @@ void LayoutManager::orderLayouts()
 {
     KConfigGroup config = Amarok::config( "Playlist Layout" );
     QString orderString = config.readEntry( "Order", "Default" );
-    
+
     QStringList knownLayouts = m_layouts.keys();
 
     QStringList orderingList = orderString.split( ';', QString::SkipEmptyParts );
 
-    foreach( QString layout, orderingList )
+    foreach( const QString &layout, orderingList )
     {
         if ( knownLayouts.contains( layout ) )
         {
-            //skip any layout names that are in config but that we dont know. Perhaps someone manually deleted a layout file
+            //skip any layout names that are in config but that we don't know. Perhaps someone manually deleted a layout file
             m_layoutNames.append( layout );
             knownLayouts.removeAll( layout );
         }
     }
 
     //now add any layouts that were not in the order config to end of list:
-    foreach( QString layout, knownLayouts )
+    foreach( const QString &layout, knownLayouts )
         m_layoutNames.append( layout );
 }
 
@@ -400,7 +472,7 @@ void Playlist::LayoutManager::storeLayoutOrdering()
 
     QString ordering;
 
-    foreach( QString name, m_layoutNames )
+    foreach( const QString &name, m_layoutNames )
     {
         ordering += name;
         ordering += ';';
@@ -408,7 +480,7 @@ void Playlist::LayoutManager::storeLayoutOrdering()
 
     if ( !ordering.isEmpty() )
         ordering.chop( 1 ); //remove trailing;
-    
+
     KConfigGroup config = Amarok::config("Playlist Layout");
     config.writeEntry( "Order", ordering );
 }
