@@ -1,7 +1,7 @@
 /****************************************************************************************
  * Copyright (c) 2007 Ian Monroe <ian@monroe.nu>                                        *
  * Copyright (c) 2008 Soren Harward <stharward@gmail.com>                               *
- * Copyright (c) 2009 Téo Mrnjavac <teo.mrnjavac@gmail.com>                             *
+ * Copyright (c) 2009 Téo Mrnjavac <teo@kde.org>                                        *
  *                                                                                      *
  * This program is free software; you can redistribute it and/or modify it under        *
  * the terms of the GNU General Public License as published by the Free Software        *
@@ -19,30 +19,36 @@
  ****************************************************************************************/
 
 #include "TrackNavigator.h"
-#include "Amarok.h"
-#include "amarokconfig.h"
+
 #include "playlist/PlaylistModelStack.h"
 
+#include "core/support/Amarok.h"
+#include "amarokconfig.h"
+#include "core/support/Debug.h"
+
 #include <QQueue>
 
 
 Playlist::TrackNavigator::TrackNavigator()
 {
     m_model = Playlist::ModelStack::instance()->top();
-    m_repeatPlaylist = ( AmarokConfig::trackProgression() == AmarokConfig::EnumTrackProgression::RepeatPlaylist );
-    connect( model(), SIGNAL( removedIds( const QList<quint64>& ) ),
-             this, SLOT( dequeueIds( const QList<quint64>& ) ) );
+
+    // Connect to the QAbstractItemModel signals of the source model.
+    //   Ignore SIGNAL dataChanged: we don't need to know when a playlist item changes.
+    //   Ignore SIGNAL layoutChanged: we don't need to know when rows are moved around.
+    connect( m_model->qaim(), SIGNAL( modelReset() ), this, SLOT( slotModelReset() ) );
+    //   Ignore SIGNAL rowsInserted.
 }
 
-bool
+void
 Playlist::TrackNavigator::queueId( const quint64 id )
 {
     QList<quint64> ids;
     ids << id;
-    return queueIds( ids );
+    queueIds( ids );
 }
 
-bool
+void
 Playlist::TrackNavigator::queueIds( const QList<quint64> &ids )
 {
     foreach( quint64 id, ids )
@@ -50,22 +56,35 @@ Playlist::TrackNavigator::queueIds( const QList<quint64> &ids )
         if( !m_queue.contains( id ) )
             m_queue.enqueue( id );
     }
+}
+
+void
+Playlist::TrackNavigator::dequeueId( const quint64 id )
+{
+    m_queue.removeAll( id );
+}
+
+bool
+Playlist::TrackNavigator::queueMoveUp( const quint64 id )
+{
+    const int idx = m_queue.indexOf( id );
+    if ( idx < 1 )
+        return false;
+    quint64 temp = m_queue[ idx - 1 ];
+    m_queue[ idx - 1 ] = m_queue[ idx ];
+    m_queue[ idx ] = temp;
     return true;
 }
 
 bool
-Playlist::TrackNavigator::dequeueId( const quint64 id )
+Playlist::TrackNavigator::queueMoveDown( const quint64 id )
 {
-    QList<quint64> ids;
-    ids << id;
-    return dequeueIds( ids );
-}
-
-bool
-Playlist::TrackNavigator::dequeueIds( const QList<quint64> &ids )
-{
-    foreach( quint64 id, ids )
-        m_queue.removeAll( id );
+    const int idx = m_queue.indexOf( id );
+    if ( idx == -1 || idx == m_queue.count() - 1 )
+        return false;
+    quint64 temp = m_queue[ idx + 1 ];
+    m_queue[ idx + 1 ] = m_queue[ idx ];
+    m_queue[ idx ] = temp;
     return true;
 }
 
@@ -80,3 +99,21 @@ QQueue<quint64> Playlist::TrackNavigator::queue()
     return m_queue;
 }
 
+void
+Playlist::TrackNavigator::slotModelReset()
+{
+    DEBUG_BLOCK
+    m_queue.clear();    // We should check 'm_model's new contents, but this is unlikely to bother anyone.
+}
+
+quint64
+Playlist::TrackNavigator::bestFallbackItem()
+{
+    quint64 item = m_model->activeId();
+
+    if ( !item )
+        if ( m_model->qaim()->rowCount() > 0 )
+            item = m_model->idAt( 0 );
+
+    return item;
+}
