@@ -15,14 +15,15 @@
  * this program.  If not, see <http://www.gnu.org/licenses/>.                           *
  ****************************************************************************************/
 
+#define DEBUG_PREFIX "AmarokLyricsScript"
+
 #include "AmarokLyricsScript.h"
 
-#include "Amarok.h"
-#include "CollectionManager.h"
-#include "Debug.h"
+#include "core/support/Amarok.h"
+#include "core/support/Debug.h"
 #include "EngineController.h"
 #include "LyricsManager.h"
-#include "Meta.h"
+#include "core/meta/Meta.h"
 #include "ScriptManager.h"
 
 #include <KApplication>
@@ -57,6 +58,7 @@ AmarokLyricsScript::showLyrics( const QString& lyrics ) const
 void
 AmarokLyricsScript::showLyricsHtml( const QString& lyrics ) const
 {
+    DEBUG_BLOCK
     Meta::TrackPtr track = The::engineController()->currentTrack();
     if( !track )
         return;
@@ -66,6 +68,7 @@ AmarokLyricsScript::showLyricsHtml( const QString& lyrics ) const
 void
 AmarokLyricsScript::showLyricsError( const QString& error ) const
 {
+    DEBUG_BLOCK
     LyricsManager::self()->lyricsError( error );
 }
 
@@ -73,6 +76,7 @@ AmarokLyricsScript::showLyricsError( const QString& error ) const
 void
 AmarokLyricsScript::showLyricsNotFound( const QString& msg ) const
 {
+    DEBUG_BLOCK
     LyricsManager::self()->lyricsNotFound( msg );
 }
  
@@ -86,10 +90,7 @@ AmarokLyricsScript::escape( const QString& str )
 void
 AmarokLyricsScript::setLyricsForTrack( const QString& trackUrl, const QString& lyrics ) const
 {
-    DEBUG_BLOCK
-    Meta::TrackPtr track = CollectionManager::instance()->trackForUrl( KUrl( trackUrl ) );
-    if( track )
-        track->setCachedLyrics( lyrics );
+    LyricsManager::self()->setLyricsForTrack( trackUrl, lyrics );
 }
 
 QString
