@@ -17,9 +17,9 @@
 
 #include "ServiceMetaBase.h"
 
-#include "Amarok.h"
-#include "Debug.h"
-#include "meta/default/DefaultMetaTypes.h"
+#include "core/support/Amarok.h"
+#include "core/support/Debug.h"
+#include "core-impl/meta/default/DefaultMetaTypes.h"
 
 using namespace Meta;
 
@@ -125,9 +125,8 @@ ServiceMetaFactory::createGenre(const QStringList & rows)
 ServiceTrack::ServiceTrack( const QString & name )
     : Meta::Track()
     , ServiceDisplayInfoProvider()
-    , CustomActionsProvider()
+    , ActionsProvider()
     , SourceInfoProvider()
-    , CurrentTrackActionsProvider()
     , BookmarkThisProvider()
     , m_provider( 0 )
     , m_genre( 0 )
@@ -136,13 +135,9 @@ ServiceTrack::ServiceTrack( const QString & name )
     , m_id( 0 )
     , m_trackNumber( 0 )
     , m_length( 0 )
-    , m_displayUrl( 0 )
-    , m_playableUrl( 0 )
-    , m_downloadableUrl( 0 )
     , m_albumId( 0 )
-    , m_albumName( 0 )
+    , m_albumName( '0' )
     , m_artistId( 0 )
-    , m_artistName( 0 )
     , m_name( name )
 {
 }
@@ -150,9 +145,8 @@ ServiceTrack::ServiceTrack( const QString & name )
 ServiceTrack::ServiceTrack( const QStringList & resultRow )
     : Meta::Track()
     , ServiceDisplayInfoProvider()
-    , CustomActionsProvider()
+    , ActionsProvider()
     , SourceInfoProvider()
-    , CurrentTrackActionsProvider()
     , BookmarkThisProvider()
     , m_provider( 0 )
     , m_genre( 0 )
@@ -219,12 +213,6 @@ ServiceTrack::name() const
     return m_name;
 }
 
-QString
-ServiceTrack::prettyName() const
-{
-    return name();
-}
-
 KUrl
 ServiceTrack::playableUrl() const
 {
@@ -348,13 +336,13 @@ ServiceTrack::setGenre( const QString &newGenre )
 }
 
 void
-ServiceTrack::setYear( const QString &newYear )
+ServiceTrack::setYear( int newYear )
 {
     Q_UNUSED( newYear )
 }
 
 /* At time of writing, the services (Jamendo & Magnatune.com) I've seen can't handle BPM yet. */
-float
+qreal
 ServiceTrack::bpm() const
 {
     return -1.0;
@@ -473,22 +461,22 @@ ServiceTrack::playCount() const
         return 0;
 }
 
-uint
+QDateTime
 ServiceTrack::lastPlayed() const
 {
     if( m_provider )
-        return m_provider->lastPlayed().toTime_t();
+        return m_provider->lastPlayed();
     else
-        return 0;
+        return QDateTime();
 }
 
-uint
+QDateTime
 ServiceTrack::firstPlayed() const
 {
     if( m_provider )
-        return m_provider->firstPlayed().toTime_t();
+        return m_provider->firstPlayed();
     else
-        return 0;
+        return QDateTime();
 }
 
 void
@@ -496,7 +484,7 @@ ServiceTrack::finishedPlaying( double playedFraction )
 {
     if( m_provider )
     {
-        m_provider->played( playedFraction );
+        m_provider->played( playedFraction, TrackPtr( this ) );
         notifyObservers();
     }
 }
@@ -539,7 +527,7 @@ ServiceTrack::setYear( YearPtr year )
 }
 
 void
-ServiceTrack::setStatisticsProvider( StatisticsProvider *provider )
+ServiceTrack::setStatisticsProvider( Statistics::StatisticsProvider *provider )
 {
     if( m_provider )
         delete m_provider;
@@ -570,12 +558,11 @@ void ServiceTrack::processInfoOf(InfoParserBase * infoParser)
 ServiceArtist::ServiceArtist( const QString & name )
     : Meta::Artist()
     , ServiceDisplayInfoProvider()
-    , CustomActionsProvider()
+    , ActionsProvider()
     , SourceInfoProvider()
     , BookmarkThisProvider()
     , m_id( 0 )
     , m_name( name )
-    , m_description( 0 )
     , m_tracks()
 {
     //nothing to do
@@ -584,7 +571,7 @@ ServiceArtist::ServiceArtist( const QString & name )
 ServiceArtist::ServiceArtist(const QStringList & resultRow)
     : Meta::Artist()
     , ServiceDisplayInfoProvider()
-    , CustomActionsProvider()
+    , ActionsProvider()
     , SourceInfoProvider()
     , BookmarkThisProvider()
     , m_tracks()
@@ -621,12 +608,6 @@ ServiceArtist::name() const
     return m_name;
 }
 
-QString
-ServiceArtist::prettyName() const
-{
-    return m_name;
-}
-
 void
 ServiceArtist::setTitle(const QString & title)
 {
@@ -676,7 +657,7 @@ ServiceArtist::processInfoOf(InfoParserBase * infoParser)
 ServiceAlbum::ServiceAlbum( const QString & name )
     : Meta::Album()
     , ServiceDisplayInfoProvider()
-    , CustomActionsProvider()
+    , ActionsProvider()
     , SourceInfoProvider()
     , BookmarkThisProvider()
     , m_id( 0 )
@@ -685,7 +666,6 @@ ServiceAlbum::ServiceAlbum( const QString & name )
     , m_isCompilation( false )
     , m_albumArtist( 0 )
     , m_artistId( 0 )
-    , m_artistName( 0 )
 {
     //nothing to do
 }
@@ -693,7 +673,7 @@ ServiceAlbum::ServiceAlbum( const QString & name )
 ServiceAlbum::ServiceAlbum(const QStringList & resultRow)
     : Meta::Album()
     , ServiceDisplayInfoProvider()
-    , CustomActionsProvider()
+    , ActionsProvider()
     , SourceInfoProvider()
     , BookmarkThisProvider()
     , m_id( resultRow[0].toInt() )
@@ -753,12 +733,6 @@ ServiceAlbum::name() const
     return m_name;
 }
 
-QString
-ServiceAlbum::prettyName() const
-{
-    return m_name;
-}
-
 void
 ServiceAlbum::setTitle(const QString & title)
 {
@@ -830,7 +804,7 @@ ServiceAlbum::processInfoOf(InfoParserBase * infoParser)
 ServiceGenre::ServiceGenre( const QString &name )
     : Meta::Genre()
     , ServiceDisplayInfoProvider()
-    , CustomActionsProvider()
+    , ActionsProvider()
     , SourceInfoProvider()
     , BookmarkThisProvider()
     , m_id( 0 )
@@ -844,7 +818,7 @@ ServiceGenre::ServiceGenre( const QString &name )
 ServiceGenre::ServiceGenre(const QStringList & row)
     : Meta::Genre()
     , ServiceDisplayInfoProvider()
-    , CustomActionsProvider()
+    , ActionsProvider()
     , SourceInfoProvider()
     , BookmarkThisProvider()
     , m_id( 0 )
@@ -877,12 +851,6 @@ ServiceGenre::name() const
     return m_name;
 }
 
-QString
-ServiceGenre::prettyName() const
-{
-    return m_name;
-}
-
 int
 ServiceGenre::albumId()
 {
@@ -920,7 +888,7 @@ ServiceGenre::processInfoOf(InfoParserBase * infoParser)
 ServiceComposer::ServiceComposer( const QString &name )
     : Meta::Composer()
     , ServiceDisplayInfoProvider()
-    , CustomActionsProvider()
+    , ActionsProvider()
     , SourceInfoProvider()
     , BookmarkThisProvider()
     , m_name( name )
@@ -940,12 +908,6 @@ ServiceComposer::name() const
     return m_name;
 }
 
-QString
-ServiceComposer::prettyName() const
-{
-    return m_name;
-}
-
 TrackList
 ServiceComposer::tracks()
 {
@@ -970,7 +932,7 @@ ServiceComposer::processInfoOf(InfoParserBase * infoParser)
 ServiceYear::ServiceYear( const QString &name )
     : Meta::Year()
     , ServiceDisplayInfoProvider()
-    , CustomActionsProvider()
+    , ActionsProvider()
     , SourceInfoProvider()
     , BookmarkThisProvider()
     , m_name( name )
@@ -990,12 +952,6 @@ ServiceYear::name() const
     return m_name;
 }
 
-QString
-ServiceYear::prettyName() const
-{
-    return m_name;
-}
-
 TrackList
 ServiceYear::tracks()
 {
