@@ -19,9 +19,7 @@
 
 #include <ServiceCollection.h>
 #include "AmpacheMeta.h"
-#include "TrackForUrlWorker.h"
-
-#include <kio/jobclasses.h>
+#include "core/collections/support/TrackForUrlWorker.h"
 
 class AmpacheTrackForUrlWorker : public Amarok::TrackForUrlWorker
 {
@@ -47,10 +45,9 @@ class AmpacheTrackForUrlWorker : public Amarok::TrackForUrlWorker
         QString m_sessionId;
 
         ServiceBase* m_service;
-
-        KIO::StoredTransferJob * m_storedTransferJob;
 };
 
+namespace Collections {
 
 /**
 A collection that dynamically fetches data from a remote location as needed
@@ -90,4 +87,7 @@ private:
 
     AmpacheTrackForUrlWorker * m_trackForUrlWorker;
 };
+
+} //namespace Collections
+
 #endif
