@@ -1,32 +1,39 @@
-/*
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * osd.cpp:   Shows some text in a pretty way independent to the WM
- * begin:     Fre Sep 26 2003
- * copyright: (C) 2004 Christian Muehlhaeuser <chris@chris.de>
- *            (C) 2004-2006 Seb Ruiz <ruiz@kde.org>
- *            (C) 2004, 2005 Max Howell
- *            (C) 2005 Gábor Lehel <illissius@gmail.com>
- *            (C) 2008 Mark Kretschmann <kretschmann@kde.org>
- */
+/****************************************************************************************
+ * Copyright (c) 2004 Christian Muehlhaeuser <chris@chris.de>                           *
+ * Copyright (c) 2004-2006 Seb Ruiz <ruiz@kde.org>                                      *
+ * Copyright (c) 2004,2005 Max Howell <max.howell@methylblue.com>                       *
+ * Copyright (c) 2005 Gabor Lehel <illissius@gmail.com>                                 *
+ * Copyright (c) 2008,2009 Mark Kretschmann <kretschmann@kde.org>                       *
+ *                                                                                      *
+ * This program is free software; you can redistribute it and/or modify it under        *
+ * the terms of the GNU General Public License as published by the Free Software        *
+ * Foundation; either version 2 of the License, or (at your option) any later           *
+ * version.                                                                             *
+ *                                                                                      *
+ * This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
+ * PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
+ *                                                                                      *
+ * You should have received a copy of the GNU General Public License along with         *
+ * this program.  If not, see <http://www.gnu.org/licenses/>.                           *
+ ****************************************************************************************/
 
 #define DEBUG_PREFIX "OSD"
 
 #include "Osd.h"
 
-#include "Amarok.h"
-#include "Debug.h"
+#include "core/support/Amarok.h"
+#include "core/support/Debug.h"
 #include "EngineController.h"
 #include "StarManager.h"
 #include "SvgHandler.h"
 #include "amarokconfig.h"
-#include "meta/MetaUtility.h"
+#include "core/meta/support/MetaUtility.h"
 
 #include <KApplication>
 #include <KIcon>
+#include <KLocale>
+#include <KWindowSystem>
 
 #include <QDesktopWidget>
 #include <QMouseEvent>
@@ -56,27 +63,35 @@ OSDWidget::OSDWidget( QWidget *parent, const char *name )
         , m_rating( 0 )
         , m_volume( 0 )
         , m_showVolume( false )
+        , m_fontScale( 1.15 )
 {
     Qt::WindowFlags flags;
     flags = Qt::WindowStaysOnTopHint | Qt::FramelessWindowHint;
     // The best of both worlds.  On Windows, setting the widget as a popup avoids a task manager entry.  On linux, a popup steals focus.
     // Therefore we go need to do it platform specific :(
+
+    //This is no longer true. Qt::Window steals focus on X11, Qt:Tool does not. Not sure if we even need the ifdefs any more...
     #ifdef Q_OS_WIN
-    flags |= Qt::Popup;
+    flags |= Qt::Tool;
     #else
-    flags |= Qt::Window | Qt::X11BypassWindowManagerHint;
+    flags |= Qt::Tool | Qt::X11BypassWindowManagerHint;
     #endif
     setWindowFlags( flags );
     setObjectName( name );
     setFocusPolicy( Qt::NoFocus );
     unsetColors();
 
-    m_timer->setSingleShot( true );
+    #ifdef Q_WS_X11
+    KWindowSystem::setType( winId(), NET::Notification );
+    #endif
 
+    m_timer->setSingleShot( true );
     connect( m_timer, SIGNAL( timeout() ), SLOT( hide() ) );
 
     //or crashes, KWindowSystem bug I think, crashes in QWidget::icon()
     kapp->setTopWidget( this );
+
+    m_volume = The::engineController()->volume();
 }
 
 OSDWidget::~OSDWidget()
@@ -99,6 +114,10 @@ OSDWidget::show( const QString &text, QImage newImage )
     else
         m_cover = Amarok::icon();
 
+    QFont f = QFont( "sans-serif" );
+    f.setPointSizeF( f.pointSizeF() * m_fontScale );
+    setFont( f );
+
     m_text = text;
     show();
 }
@@ -132,7 +151,7 @@ OSDWidget::volumeChanged( int volume )
         QString muteState = "";
         m_showVolume = true;
 
-        m_text = i18n("Volume: %1% %2", m_volume, ( The::engineController()->isMuted() ? "(muted)" : "" ) );
+        m_text = i18n("Volume: %1% %2", m_volume, ( The::engineController()->isMuted() ? i18n("(muted)") : "" ) );
 
         show();
     }
@@ -166,7 +185,7 @@ OSDWidget::setVisible( bool visible )
 
         update();
     }
-    else 
+    else
         QWidget::setVisible( visible );
 }
 
@@ -321,6 +340,7 @@ OSDWidget::paintEvent( QPaintEvent *e )
 
     if( !m_showVolume && m_rating > 0 && !m_paused )
     {
+        // TODO: Check if we couldn't use a KRatingPainter instead
         QPixmap* star = StarManager::instance()->getStar( m_rating/2 );
         QRect r( rect );
 
@@ -363,10 +383,7 @@ OSDWidget::paintEvent( QPaintEvent *e )
     }
     p.setPen( palette().color( QPalette::Active, QPalette::WindowText ) );
     //p.setPen( Qt::white ); // This too.
-    p.setFont( font() );
     p.drawText( rect, align, m_text );
-
-    m_paused = false;
 }
 
 void
@@ -425,14 +442,11 @@ OSDPreviewWidget::OSDPreviewWidget( QWidget *parent )
         , m_dragging( false )
 {
     setObjectName( "osdpreview" );
-    m_text = i18n( "OSD Preview - drag to reposition" );
+    m_text = i18n( "On-Screen-Display preview\nDrag to reposition" );
     m_duration = 0;
     m_alignment = static_cast<Alignment>( AmarokConfig::osdAlignment() );
     m_y = AmarokConfig::osdYOffset();
     m_cover = Amarok::icon();
-    QFont f = font();
-    f.setPointSize( 16 );
-    setFont( f );
     setTranslucent( AmarokConfig::osdUseTranslucency() );
     setText( m_text );
     setImage( m_cover );
@@ -547,9 +561,32 @@ Amarok::OSD::destroy()
 
 Amarok::OSD::OSD()
     : OSDWidget( 0 )
-    , EngineObserver( The::engineController() )
 {
     s_instance = this;
+
+    EngineController* const engine = The::engineController();
+
+    if( engine->isPlaying() )
+        trackPlaying( engine->currentTrack() );
+
+    connect( engine, SIGNAL( trackPlaying( Meta::TrackPtr ) ),
+             this, SLOT( trackPlaying( Meta::TrackPtr ) ) );
+    connect( engine, SIGNAL( stopped( qint64, qint64 ) ),
+             this, SLOT( stopped() ) );
+    connect( engine, SIGNAL( paused() ),
+             this, SLOT( paused() ) );
+
+    connect( engine, SIGNAL( trackMetadataChanged( Meta::TrackPtr ) ),
+             this, SLOT( metadataChanged() ) );
+    connect( engine, SIGNAL( albumMetadataChanged( Meta::AlbumPtr ) ),
+             this, SLOT( metadataChanged() ) );
+
+    connect( engine, SIGNAL( volumeChanged( int ) ),
+             this, SLOT( volumeChanged( int ) ) );
+
+    connect( engine, SIGNAL( muteStateChanged( bool ) ),
+             this, SLOT( muteStateChanged( bool ) ) );
+
 }
 
 Amarok::OSD::~OSD()
@@ -563,8 +600,10 @@ Amarok::OSD::show( Meta::TrackPtr track ) //slot
 
     QString text;
     if( !track || track->playableUrl().isEmpty() )
+    {
         text = i18n( "No track playing" );
-
+        setRating( 0 ); // otherwise stars from last rating change are visible
+    }
     else
     {
         setRating( track->rating() );
@@ -576,7 +615,7 @@ Amarok::OSD::show( Meta::TrackPtr track ) //slot
         else
             text += '\n';
         if( track->length() > 0 )
-            text += Meta::secToPrettyTime( track->length() );
+            text += Meta::msToPrettyTime( track->length() );
     }
 
     if( text.isEmpty() )
@@ -590,7 +629,7 @@ Amarok::OSD::show( Meta::TrackPtr track ) //slot
 
     QImage image;
     if( track && track->album() )
-        image = track->album()->imageWithBorder( 100, 5 ).toImage();
+        image = The::svgHandler()->imageWithBorder( track->album(), 100, 5 ).toImage();
 
     OSDWidget::show( text, image );
 }
@@ -605,7 +644,7 @@ Amarok::OSD::applySettings()
     setEnabled( AmarokConfig::osdEnabled() );
     setOffset( AmarokConfig::osdYOffset() );
     setScreen( AmarokConfig::osdScreen() );
-    setFont( AmarokConfig::osdFont() );
+    setFontScale( AmarokConfig::osdFontScaling() );
 
     if( AmarokConfig::osdUseCustomColors() )
         setTextColor( AmarokConfig::osdTextColor() );
@@ -630,13 +669,7 @@ Amarok::OSD::forceToggleOSD()
 }
 
 void
-Amarok::OSD::engineVolumeChanged( int newVolume )
-{
-    volumeChanged( newVolume );
-}
-
-void
-Amarok::OSD::engineMuteStateChanged( bool mute )
+Amarok::OSD::muteStateChanged( bool mute )
 {
     Q_UNUSED( mute )
 
@@ -644,40 +677,36 @@ Amarok::OSD::engineMuteStateChanged( bool mute )
 }
 
 void
-Amarok::OSD::engineStateChanged( Phonon::State state, Phonon::State oldState )
+Amarok::OSD::trackPlaying( Meta::TrackPtr track )
 {
-    Q_UNUSED( oldState )
-    DEBUG_BLOCK
+    m_currentTrack = track;
 
-    Meta::TrackPtr track = The::engineController()->currentTrack();
-
-    switch( state )
-    {
-        case Phonon::PlayingState:
-            unsubscribeFrom( m_currentTrack );
-            m_currentTrack = track;
-            subscribeTo( track );
-            metadataChanged( track );
-            m_paused = false;
-            break;
-
-        case Phonon::PausedState:
-            setImage( QImage( KIconLoader::global()->iconPath( "amarok", -KIconLoader::SizeHuge ) ) );
-            OSDWidget::show( i18n( "Paused" ) );
-            m_paused = true;
-            break;
-
-        default:
-            break;
-    }
+    m_paused = false;
+    show( m_currentTrack );
 }
 
 void
-Amarok::OSD::metadataChanged( Meta::TrackPtr track )
+Amarok::OSD::stopped()
 {
-    Q_UNUSED( track )
-    DEBUG_BLOCK
+    setImage( QImage( KIconLoader::global()->iconPath( "amarok", -KIconLoader::SizeHuge ) ) );
+    setRating( 0 ); // otherwise stars from last rating change are visible
+    OSDWidget::show( i18n( "Stopped" ) );
+    m_paused = false;
+}
 
+void
+Amarok::OSD::paused()
+{
+    setImage( QImage( KIconLoader::global()->iconPath( "amarok", -KIconLoader::SizeHuge ) ) );
+    setRating( 0 ); // otherwise stars from last rating change are visible
+    OSDWidget::show( i18n( "Paused" ) );
+    m_paused = true;
+}
+
+void
+Amarok::OSD::metadataChanged()
+{
+    // this also covers all cases where a stream get's new metadata.
     show( m_currentTrack );
 }
 
@@ -769,4 +798,3 @@ namespace ShadowEngine
 }
 
 #include "Osd.moc"
-
