@@ -16,7 +16,7 @@
  
 #include "Splitter.h"
 
-#include "Debug.h"
+#include "core/support/Debug.h"
 #include "SvgHandler.h"
 
 #include <QPainter>
